# -*- coding: utf-8 -*-

import json

from unittest import TestCase
from unittest.mock import Mock, MagicMock, patch

from vault.tests.fakes import fake_request, UserFactory
from identity.tests.fakes import FakeToken
from vault.views import (DashboardView, SetProjectView, DeleteUserTeamView,
                         AddUserTeamView, ListUserTeamView, UpdateTeamsUsersView)
from django.utils.translation import gettext as _
from django.contrib.auth.models import Group, User


class BaseTestCase(TestCase):

    def setUp(self):
        self.request = fake_request()
        self.anonymous_request = fake_request(user=False)

    def tearDown(self):
        User.objects.all().delete()
        Group.objects.all().delete()

    @classmethod
    def setUpClass(cls):
        patch('identity.keystone.v3').start()

    @classmethod
    def tearDownClass(cls):
        patch.stopall()


class DashboardTest(TestCase):

    def setUp(self):
        self.view = DashboardView.as_view()
        self.request = fake_request(method='GET')

        # does not connect to the keystone client
        patch('keystoneclient.v3.client.Client').start()

    def tearDown(self):
        patch.stopall()

    def test_dashboard_needs_authentication(self):
        req = fake_request(method='GET', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_show_dashboard(self):
        response = self.view(self.request)
        self.assertEqual(response.status_code, 200)

        response.render()
        self.assertIn(b'Dashboard', response.content)


class SetProjectTest(BaseTestCase):

    def setUp(self):
        super(SetProjectTest, self).setUp()

        self.view = SetProjectView.as_view()

    def test_set_project_needs_authentication(self):
        response = self.view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    @patch('vault.views.switch')
    def test_set_new_project_id_to_session(self, mock_switch):
        self.assertEqual(self.request.session.get('project_id'), '1')

        response = self.view(self.request, project_id=2)
        self.assertEqual(self.request.session.get('project_id'), 2)

    @patch('vault.views.switch')
    def test_set_new_project_id_to_session_exception(self, mock_switch):
        mock_switch.side_effect = ValueError()

        self.assertEqual(self.request.session.get('project_id'), '1')

        response = self.view(self.request, project_id=2)
        self.assertEqual(self.request.session.get('project_id'), 2)

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(msgs[0].message, _('Unable to change your project.'))


class TestDeleteUserTeam(BaseTestCase):
    view_class = DeleteUserTeamView

    def setUp(self):
        super(TestDeleteUserTeam, self).setUp()

        self.view = self.view_class.as_view()

        self.request.method = 'POST'

        self.group = Group(id=1, name="teamtest")
        self.group.save()

        self.user = User(id=1, username="usertest")
        self.user.save()

    def test_delete_user_and_team_flow_needs_authentication(self):
        response = self.view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_delete_user_and_team_flow_dont_need_to_be_superuser(self):
        self.request.user.is_superuser = False

        post = self.request.POST.copy()
        post.setdefault('group', int('1'))
        post.setdefault('user', int('1'))

        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'{"msg": "ok"}')

    def test_delete_user_and_team_flow_work_fine(self):
        self.request.user.is_superuser = False

        post = self.request.POST.copy()
        post.setdefault('group', int('1'))
        post.setdefault('user', int('1'))

        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'{"msg": "ok"}')

    def test_delete_user_and_team_flow_fail(self):
        self.request.user.is_superuser = False

        post = self.request.POST.copy()
        post.setdefault('group', int('10'))
        post.setdefault('user', int('1'))

        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)


class TestListUserTeam(BaseTestCase):
    view_class = ListUserTeamView

    def setUp(self):
        super(TestListUserTeam, self).setUp()

        self.view = self.view_class.as_view()

        self.request.method = 'POST'

        self.group = Group(id=1, name="teamtest")
        self.group.save()

        self.user = User(id=1, username="usertest")
        self.user.save()

    def test_list_users_teams_flow_needs_authentication(self):
        req = fake_request(user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_list_users_teams_flow_dont_need_to_be_superuser(self):
        self.request.user.is_superuser = False
        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)

    def test_list_users_teams_flow_work_fine(self):
        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.reason_phrase, 'OK')

    @patch('django.contrib.auth.models.Group.objects.get')
    def test_list_users_teams_flow_fail(self, mock_get):
        mock_get.return_value = None
        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)


class TestAddUserTeam(BaseTestCase):
    view_class = AddUserTeamView

    def setUp(self):
        super(TestAddUserTeam, self).setUp()

        self.view = self.view_class.as_view()

        self.request.method = 'POST'
        self.anonymous_request.method = 'POST'

        self.group = Group(id=1, name="teamtest")
        self.group.save()

        self.user = User(id=1, username="usertest")
        self.user.save()

    def test_add_user_and_team_flow_needs_authentication(self):
        response = self.view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_add_user_and_team_flow_from_user_not_superuser_work_fine(self):
        self.request.user.is_superuser = False

        post = self.request.POST.copy()
        post.setdefault('group', int('1'))
        post.setdefault('user', int('1'))

        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)

    def test_add_user_and_team_flow_from_user_already_registered(self):
        self.request.user.is_superuser = False

        post = self.request.POST.copy()
        post.setdefault('group', int('1'))
        post.setdefault('user', int('1'))

        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'{"msg": "ok"}')

        response2 = self.view(self.request)

        self.assertEqual(response2.status_code, 500)

    def test_add_user_and_team_flow_from_user_fail(self):
        req = fake_request(method='POST')
        req.user.is_superuser = False

        post = req.POST.copy()
        post.setdefault('group', int('10'))
        post.setdefault('user', int('1'))

        req.POST = post

        response = self.view(req)

        self.assertEqual(response.status_code, 500)


class UpdateTeamsUsersTest(BaseTestCase):

    def setUp(self):
        super(UpdateTeamsUsersTest, self).setUp()

        self.view = UpdateTeamsUsersView.as_view()

        self.roles = {
            "roles": {
                "driver_processado": [
                    {"1751": "qa2"},
                    {"11": "Storm"}
                ],
                "sheets": [
                    {"1754": "Financeiro"},
                    {"11": "Storm"}
                ],
                "reports": [
                    {"11": "Storm"}
                ]
            }
        }

        self.request.META.update({
            'SERVER_NAME': 'globo.com',
            'SERVER_PORT': '80',
            'HTTP_HOST': 'localhost'
        })

        # Silent log
        mock_log = patch('vault.views.log').start()

        # does not connect to the keystone client
        # patch('keystoneclient.v2_0.client.Client').start()

        group = Group(id=1, name="teamtest")
        group.save()

        # Usuario do time "teamtest"
        user = User(id=1, username="usertest-1")
        user.save()

        user.groups.add(group)
        user.save()

        # Usuario sem time, para ser adicionado ao time "teamtest"
        user = User(id=2, username="usertest-2")
        user.save()

    def test_manage_users_teams_flow_needs_authentication(self):
        response = self.view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_manage_users_teams_flow_ensure_user_list_was_filled_up(self):
        response = self.view(self.request)
        response.render()

        self.assertIn('<option value="2">usertest-2</option>', response.content.decode('utf-8'))

    def test_manage_users_teams_flow_ensure_group_list_was_filled_up(self):
        response = self.view(self.request)
        response.render()
        group = self.request.user.groups.first()

        self.assertIn('<option value="{}">{}'.format(group.id, group.name),
                      response.content.decode('utf-8'))

    def test_manage_users_teams_flow_for_a_user_not_superuser_work_fine(self):
        self.request.user.is_superuser = False
        response = self.view(self.request)

        self.assertEqual(response.status_code, 200)

    @patch('django.contrib.auth.models.Group.objects.all')
    def test_manage_users_teams_fail_to_get_user_group(self, mock_all):
        mock_all.return_value = None
        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Audit',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True)),
                ('user', models.CharField(max_length=60, null=True, blank=True)),
                ('action', models.CharField(max_length=60, null=True, blank=True)),
                ('item', models.TextField(max_length=255, null=True, blank=True)),
                ('through', models.TextField(default='vault', max_length=255)),
                ('created_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'audit',
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-

""" Backup interface views. """

import json
import logging
import requests
import time

from django.conf import settings
from django.contrib import messages
from django.http import HttpResponse
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.decorators import login_required

from keystoneclient import exceptions

from storage.models import BackupContainer
from storage.utils import get_token_id, get_storage_endpoint
from vault import utils
from identity.keystone import Keystone, exceptions
from actionlogger.actionlogger import ActionLogger


log = logging.getLogger(__name__)
actionlog = ActionLogger()


def _check_backup_user(request, project_id):
    try:
        keystone = Keystone(request)
    except exceptions.AuthorizationFailure:
        msg = _('Unable to retrieve Keystone data')
        messages.add_message(request, messages.ERROR, msg)
        log.error(f'{request.user}: {msg}')

        return False

    if keystone.conn is None:
        log.error('check_backup_user: Keystone connection error')
        return False

    all_users = keystone.user_list()
    backup_user = None
    for user in all_users:
        if user.username == settings.BACKUP_USER:
            backup_user = user

    if backup_user is None:
        log.error('check_backup_user: Undefined backup user')
        return False

    all_roles = keystone.role_list()
    backup_role = None
    for role in all_roles:
        if role.name == settings.BACKUP_USER_ROLE:
            backup_role = role

    if backup_role is None:
        log.error('check_backup_user: Undefined backup role')
        return False

    items = BackupContainer.objects.filter(project_id=project_id)
    if items.count() == 0:
        keystone.remove_user_role(project=project_id,
                                  role=backup_role,
                                  user=backup_user)
        return True

    try:
        keystone.add_user_role(project=project_id,
                               role=backup_role,
                               user=backup_user)
    except exceptions.Conflict:
        log.info('backup_user already with role'.format(project_id))

    return True


def _enable_backup(container, project_id, project_name):
    create_url = 'http://{}:{}@{}/config/create'.format(
        settings.BACKUP_API_USER,
        settings.BACKUP_API_PASSWORD,
        settings.BACKUP_API_URL
    )

    try:
        req = requests.post(create_url, json={
            'name': '{}_{}'.format(project_name, container),
            'type': 'swift',
            'parameters': {
                'endpoint_type': 'admin',
                'env_auth': 'true',
                'tenant': project_name
            }
        })
    except Exception as err:
        log.error('Enable backup error: {}'.format(err))
        return False

    items = BackupContainer.objects.filter(container=container,
                                           project_id=project_id)
    if items.count() > 0:
        return True

    try:
        item = BackupContainer(container=container,
                               project_id=project_id,
                               project_name=project_name)
        item.save()
    except Exception as err:
        log.error('Enable backup error: {}'.format(err))
        return False

    return True


def _disable_backup(container, project_id, project_name):
    delete_url = 'http://{}:{}@{}/config/delete'.format(
        settings.BACKUP_API_USER,
        settings.BACKUP_API_PASSWORD,
        settings.BACKUP_API_URL
    )
    try:
        req = requests.post(delete_url, json={
            'name': '{}_{}'.format(project_name, container)
        })
    except Exception as err:
        log.error('Disable backup error: {}'.format(err))
        return False

    items = BackupContainer.objects.filter(container=container,
                                           project_id=project_id)
    if items.count() == 0:
        return True

    try:
        items[0].delete()
    except Exception as err:
        log.error('Disable backup error: {}'.format(err))
        return False

    return True


@utils.project_required
@login_required
def backup_restore(request, project):
    container = request.POST.get("container")
    project_name = request.POST.get("project_name")
    backup_type = request.POST.get("backup_type")

    content = {'message': ''}
    status = 200

    restore_url = 'http://{}:{}@{}/sync/copy'.format(
        settings.BACKUP_API_USER,
        settings.BACKUP_API_PASSWORD,
        settings.BACKUP_API_URL
    )

    if backup_type == 'daily':
        s3_bucket = 'swift-backup-daily'
    elif backup_type == 'weekly':
        s3_bucket = 'swift-backup-weekly'
    else:
        s3_bucket = 'swift-backup-monthly'

    name = '{}_{}'.format(project_name, container)
    ts = int(time.time())
    restore_container = '{}_{}_{}'.format(container, backup_type, ts)
    content['message'] = 'Backup restaurado no container: {}'.format(restore_container)

    try:
        req = requests.post(restore_url, json={
            "srcFs": "amazon:/{}/{}/{}".format(s3_bucket, project_name, container),
            "dstFs": "{}:/{}".format(name, restore_container),
            "_async": True
        })
        result = req.json()
        content['job'] = result['jobid']
    except Exception as err:
        log.error('Restore backup error: {}'.format(err))
        content['message'] = 'Restore backup error: {}'.format(err)
        return HttpResponse(json.dumps(content),
                            content_type='application/json',
                            status=500)

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=200)


def check_backup_conditions(request, container):
    backup_object_count_value = int(settings.BACKUP_OBJECT_COUNT_VALUE)
    backup_object_bytes_value = int(settings.BACKUP_OBJECT_BYTES_VALUE)
    storage_url = get_storage_endpoint(request, 'adminURL')
    headers = {'X-Storage-Token': get_token_id(request)}

    url = '{0}/{1}'.format(storage_url, container)

    response = requests.head(url, headers=headers,
                             verify=not settings.SWIFT_INSECURE)

    if int(response.headers['X-Container-Object-Count']) >= backup_object_count_value:
        return False, _('Error when activating container backup. Container cannot contain more than {} objects').format(backup_object_count_value)

    if int(response.headers['X-Container-Bytes-Used']) >= backup_object_bytes_value:
        return False, _('Error when activating container backup. Container cannot contain more than {}').format(backup_object_count_value)

    return True, 'Success'


@utils.project_required
@login_required
def config_backup_container(request, project, container):
    action = request.GET.get('status')
    if action is None and action not in ['enabled', 'disabled']:
        return HttpResponse(
            json.dumps({'message': _('Wrong "status" parameter')}),
            content_type='application/json',
            status=400
        )

    project_id = request.session.get('project_id')
    project_name = request.session.get('project_name')

    status = 200
    content = {'message': ''}
    result = False
    conditions, msg = check_backup_conditions(request, container)

    if conditions:
        if action == 'enabled':
            result = _enable_backup(container, project_id, project_name)
            msg = '{} "{}"'.format(_('Backup enabled for container'), container)

        if action == 'disabled':
            result = _disable_backup(container, project_id, project_name)
            msg = '{} "{}"'.format(_('Backup disabled for container'), container)

        if result:
            check_ok = _check_backup_user(request, project_id)
            if not check_ok:
                _disable_backup(container, project_id, project_name)
                status = 500
                msg = _("Backup status error: can't set backup user permission")
                log.error('{}. Project: {}, Container: {}'.format(msg,
                                                                  project_name,
                                                                  container))
        else:
            status = 500
            msg = _('Error when updating container backup status')
            log.error('{}. Project: {}, Container: {}'.format(msg,
                                                              project_name,
                                                              container))
    else:
        status = 412

    content['message'] = msg

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)


def get_current_backup(container, project_id):
    if not settings.BACKUP_ENABLED:
        return None

    query = BackupContainer.objects.filter(container=container,
                                           project_id=project_id)
    if query.count() > 0:
        return query[0]

    return None


@utils.project_required
@login_required
def container_backup_status(request, project, container):
    project_id = request.session.get('project_id')
    status, content = 200, {'status': 'disabled'}

    if get_current_backup(container, project_id):
        content['status'] = 'enabled'

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)


# storage API
def container_backup_list(request):
    status, content = 200, []
    items = BackupContainer.objects.all()

    if items.count() > 0:
        content = [{"container": i.container,
                    "project_id": i.project_id,
                    "project_name": i.project_name} for i in items]
    else:
        status = 404
        log.info(_("Can't find any container to backup"))

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)



# -*- coding: utf-8 -*-

import logging
import json

from django.http import HttpResponse

log = logging.getLogger(__name__)


class JsonInfo:

    def __init__(self, *args, **kwargs):

        self.request = kwargs["request"]
        self._menu = {}
        self._widgets = []

    @property
    def menu_info(self):
        self._menu = self.generate_menu_info()
        status = 500 if "error" in self._menu else 200
        if status == 500:
            log.error('Error on {}\'s menu info: {}'.format(
                    type(self).__name__,
                    self._menu["error"]))
        content = self._menu
        return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)

    @property
    def widget_info(self):
        self.validate_widget_info()
        status = 500 if "error" in self._widgets else 200
        if status == 500:
            log.error('Error on {}\'s widget info: {}'.format(
                    type(self).__name__,
                    self._widgets["error"]))
        content = self._widgets
        return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)

    @property
    def all_info(self):
        self._menu = self.generate_menu_info()
        self.validate_widget_info()
        status = 200
        if "error" in self._menu:
            status = 500
            log.error('Error on {}\'s menu info: {}'.format(
                    type(self).__name__,
                    self._menu["error"]))
        if "error" in self._widgets:
            status = 500
            log.error('Error on {}\'s widget info: {}'.format(
                    type(self).__name__,
                    self._widgets["error"]))
        content = {
            "menu": self._menu,
            "widgets": self._widgets
        }
        return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)

    def generate_menu_info(self):
        # Method must be overridden
        return {}

    def generate_widget_info(self):
        # Method must be overridden
        return []

    def translate_color(self, color):
        colors = {
            "blue": "#3e95cc",
            "purple": "#8b40a9",
            "red": "#cc543f",
            "orange": "#e6762c",
            "yellow": "#f5bc00",
            "green": "#688f10",
            "cyan": "#29cac1",
            "pink": "#df6c98",
            "brown": "#8e4b10",
            "gray": "#757575"
        }

        return colors.get(color, colors["gray"])

    def validate_widget_info(self):
        self._widgets = self.generate_widget_info()
        for widget in self._widgets:
            widget["type"] = widget.get("type", "default")
            widget["name"] = widget.get("name", type(self).__name__)
            widget["title"] = widget.get("title", "")
            widget["subtitle"] = widget.get("subtitle",
                    type(self).__module__.split('.')[0])
            widget["color"] = self.translate_color(widget.get("color"))
            widget["icon"] = widget.get("icon", "far fa-question-circle")
            widget["url"] = widget.get("url", "/")
            widget["properties"] = widget.get("properties", [])
            widget["buttons"] = widget.get("buttons", [])
            for prop in widget["properties"]:
                prop["name"] = prop.get("name", "")
                prop["description"] = prop.get("description", "")
                prop["value"] = prop.get("value", "")
            for button in widget["buttons"]:
                button["name"] = button.get("name", "")
                button["url"] = button.get("url", "/")

    def render(self, request):
        option = request.GET.get("opt", "all").lower()
        if option == "menu":
            return self.menu_info
        elif option == "widgets":
            return self.widget_info
        elif option == "all":
            return self.all_info
        else:
            status = 400
            content = {
                "error": "Unknown option \"{}\"".format(option)
            }
            return HttpResponse(json.dumps(content),
                            content_type='application/json',
                            status=status)



from django.forms.widgets import PasswordInput


class PasswordInputWithEye(PasswordInput):

    def __init__(self, attrs=None, render_value=False):
        super().__init__(attrs=attrs, render_value=render_value)
        self.template_name = 'vault/forms/widgets/password.html'



# -*- coding: utf-8 -*-

from unittest.mock import patch, Mock
from unittest import TestCase
import requests

from swiftclient import client

from django.urls import reverse
from django.utils.translation import gettext as _
from django.test.utils import override_settings
from django.contrib.auth.models import Group, User

from storage.tests import fakes
from storage import views

from vault.tests.fakes import fake_request
from vault import utils

class MockRequest:
    def __init__(self):
        self.text = '{}'

class BaseTestCase(TestCase):

    def setUp(self):
        self.request = fake_request()
        self.anonymous_request = fake_request(user=False)
        patch('storage.views.main.actionlog',
              Mock(return_value=None)).start()

    def tearDown(self):
        User.objects.all().delete()
        Group.objects.all().delete()

    @classmethod
    def setUpClass(cls):
        patch('identity.keystone.v3').start()

    @classmethod
    def tearDownClass(cls):
        patch.stopall()


class TestStorage(BaseTestCase):

    def test_containerview_needs_authentication(self):
        response = views.containerview(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_create_container_needs_authentication(self):
        response = views.create_container(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_objectview_needs_authentication(self):
        response = views.objectview(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_delete_container_view_needs_authentication(self):
        response = views.delete_container_view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_create_object_needs_authentication(self):
        response = views.create_object(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_upload_needs_authentication(self):
        response = views.upload(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_delete_object_view_needs_authentication(self):
        response = views.delete_object_view(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_delete_pseudofolder_needs_authentication(self):
        response = views.delete_pseudofolder(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_create_pseudofolder_needs_authentication(self):
        response = views.create_pseudofolder(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_object_versioning_needs_authentication(self):
        response = views.object_versioning(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    def test_remove_from_cache_needs_authentication(self):
        response = views.remove_from_cache(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    @patch('storage.views.main.client.get_account')
    def test_containerview_redirect_to_dashboard_without_project_in_session(self, mock_get_account):
        mock_get_account.return_value = fakes.get_account()
        self.request.session['project_id'] = None

        response = views.containerview(self.request, None)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse('change_project'))

    @patch('requests.get')
    @patch('storage.views.main.client.get_account')
    def test_containerview_list_containters(self, mock_get_account, mock_get):
        mock_get_account.return_value = fakes.get_account()
        mock_get.return_value = fakes.FakeRequestResponse(200, headers={"get": {"X-Bla": "Bla", "X-Ble": "Ble"}})
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        project_name = self.request.session.get('project_name')
        response = views.containerview(self.request, project_name)

        self.assertEqual(response.status_code, 200)

        expected = '/p/{}/storage/objects/container1/'.format(project_name)
        self.assertIn(expected, response.content.decode('UTF-8'))

        expected = '/p/{}/storage/objects/container2/'.format(project_name)
        self.assertIn(expected, response.content.decode('UTF-8'))

        expected = '/p/{}/storage/objects/container3/'.format(project_name)
        self.assertIn(expected, response.content.decode('UTF-8'))

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.get_account')
    def test_containerview_clientexception(self, mock_get_account, mock_logging):
        mock_get_account.side_effect = client.ClientException('')
        project_name = self.request.session.get('project_name')
        self.request.META.update({'HTTP_HOST': 'localhost'})

        views.containerview(self.request, project_name)

        msgs = [msg for msg in self.request._messages]

        self.assertTrue(mock_get_account.called)
        self.assertTrue(mock_logging.called)

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Unable to list containers'))

    @patch('storage.views.main.client.get_container')
    def test_objectview_list_objects(self, mock_get_container):
        mock_get_container.return_value = fakes.get_container()
        project_name = self.request.session.get('project_name')

        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })

        response = views.objectview(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)

        expected = '/AUTH_1/fakecontainer/obj_pelo_vip.html'
        self.assertIn(expected, response.content.decode('UTF-8'))

        # Botao de views.upload File
        self.assertIn('/p/{}/storage/upload/fakecontainer/'.format(project_name), response.content.decode('UTF-8'))

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.get_container')
    def test_objectview_clientexception(self, mock_get_container, mock_logging):
        mock_get_container.side_effect = client.ClientException('')
        project_name = self.request.session.get('project_name')
        views.objectview(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertTrue(mock_get_container.called)
        self.assertTrue(mock_logging.called)

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Access denied'))

    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.put_container')
    def test_create_container_valid_form(self, mock_put_container, mock_logging, mock_log):
        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({'containername': 'fakecontainer'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        views.create_container(self.request, project_name)
        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Container created'))
        self.assertTrue(mock_put_container.called)
        self.assertFalse(mock_logging.called)

        user = self.request.user.username
        mock_log.assert_called_with(user, "create", "fakecontainer")

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.put_container')
    def test_create_container_invalid_form(self, mock_put_container, mock_logging):

        mock_put_container.side_effect = client.ClientException('')

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()

        post.update({'containername': ''})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.create_container(self.request, project_name)
        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 0)
        self.assertFalse(mock_put_container.called)
        self.assertFalse(mock_logging.called)

        self.assertEqual(response.status_code, 200)
        self.assertIn(_('This field is required.'), response.content.decode('UTF-8'))

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.put_container')
    def test_create_container_invalid_container_names(self, mock_put_container, mock_logging):

        mock_put_container.side_effect = client.ClientException('')

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()

        post.update({'containername': '.'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.create_container(self.request, project_name)
        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 0)
        self.assertFalse(mock_put_container.called)
        self.assertFalse(mock_logging.called)

        self.assertEqual(response.status_code, 200)
        self.assertIn('Enter a valid name consisting of letters, numbers, underscores or hyphens.', response.content.decode('UTF-8'))

        post.update({'containername': '..'})
        self.request.POST = post
        response = views.create_container(self.request, project_name)
        self.assertIn('Enter a valid name consisting of letters, numbers, underscores or hyphens.', response.content.decode('UTF-8'))

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.put_container')
    def test_create_container_fail_to_create(self, mock_put_container, mock_logging):

        mock_put_container.side_effect = client.ClientException('')

        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({'containername': 'fakecontainer'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        views.create_container(self.request, project_name)
        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Access denied'))
        self.assertTrue(mock_put_container.called)
        self.assertTrue(mock_logging.called)

    @patch('storage.views.main.requests.put')
    def test_create_object_status_201(self, mock_requests_put):
        mock_requests_put.return_value = fakes.FakeRequestResponse(201)
        self.request.FILES['file1'] = fakes.get_temporary_text_file()

        prefix = ''
        fakecontainer = 'fakecontainer'
        project_name = self.request.session.get('project_name')

        response = views.create_object(self.request, project_name, fakecontainer)

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Object created'))
        self.assertTrue(mock_requests_put.called)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': prefix,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.requests.put')
    def test_create_object_status_201_with_prefix(self, mock_requests_put):
        mock_requests_put.return_value = fakes.FakeRequestResponse(201)
        self.request.FILES['file1'] = fakes.get_temporary_text_file()

        prefix = 'prefix/'
        fakecontainer = 'fakecontainer'
        project_name = self.request.session.get('project_name')

        response = views.create_object(self.request, project_name, fakecontainer, prefix)

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Object created'))
        self.assertTrue(mock_requests_put.called)

        # Transforma em string toda a chamada do Mock, pois nao foi possivel
        #  pegar os argumentos posicionais para validar
        str_call = str(mock_requests_put.call_args)
        self.assertIn('/fakecontainer/prefix/foo.txt', str_call)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': prefix,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.requests.put')
    def test_create_object_status_401(self, mock_requests_put, mock_logging):
        mock_requests_put.return_value = fakes.FakeRequestResponse(401)
        self.request.FILES['file1'] = fakes.get_temporary_text_file()
        project_name = self.request.session.get('project_name')

        views.create_object(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Access denied'))
        self.assertTrue(mock_requests_put.called)
        self.assertFalse(mock_logging.called)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.requests.put')
    def test_create_object_status_403(self, mock_requests_put, mock_logging):
        mock_requests_put.return_value = fakes.FakeRequestResponse(403)
        self.request.FILES['file1'] = fakes.get_temporary_text_file()
        project_name = self.request.session.get('project_name')

        views.create_object(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Access denied'))
        self.assertTrue(mock_requests_put.called)
        self.assertFalse(mock_logging.called)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.requests.put')
    def test_create_object_status_other_than_above(self, mock_requests_put, mock_logging):
        mock_requests_put.return_value = fakes.FakeRequestResponse(404)
        self.request.FILES['file1'] = fakes.get_temporary_text_file()
        project_name = self.request.session.get('project_name')

        views.create_object(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)

        expected = msg = 'Fail to create object ({0}).'.format(404)
        self.assertEqual(msgs[0].message, expected)
        self.assertTrue(mock_requests_put.called)
        self.assertFalse(mock_logging.called)

    @patch('storage.views.main.delete_container')
    def test_delete_container_view_deletes_with_success(self, mock_delete_container):
        mock_delete_container.return_value = True
        self.request.method = 'DELETE'
        project_name = self.request.session.get('project_name')
        response = views.delete_container_view(self.request, project_name, 'container')

        self.assertTrue(mock_delete_container.called)
        self.assertEqual(response.status_code, 200)
        self.assertIn(_('Container deleted'), response.content.decode('unicode-escape'))

    @patch('storage.views.main.delete_container')
    def test_delete_container_view_deletes_with_failure(self, mock_delete_container):
        mock_delete_container.return_value = False
        self.request.method = 'DELETE'
        project_name = self.request.session.get('project_name')
        response = views.delete_container_view(self.request, project_name, 'container')

        self.assertTrue(mock_delete_container.called)
        self.assertEqual(response.status_code, 500)
        self.assertIn(_('Container delete error'), response.content.decode('UTF-8'))

    @patch('storage.views.main.client.get_object')
    @patch('storage.views.main.client.delete_object')
    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.client.delete_container')
    def test_delete_container_without_deleting_objects(self,
                                                       mock_delete_container,
                                                       mock_action_log,
                                                       mock_delete_object,
                                                       mock_get_object):
        fakecontainer = 'fakecontainer'
        resp = views.delete_container(self.request, fakecontainer, force=False)
        self.assertTrue(resp)

        self.assertFalse(mock_get_object.called)
        self.assertFalse(mock_delete_object.called)

        self.assertTrue(mock_delete_container.called)
        self.assertTrue(mock_action_log.called)

    @patch('storage.views.main.client.head_container')
    @patch('storage.views.main.client.get_container')
    @patch('storage.views.main.get_info')
    @patch('storage.views.main.prepare_data_name')
    @patch('storage.views.main.client.delete_object')
    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.client.delete_container')
    def test_delete_container_deleting_objects(self, mock_delete_container,
                                               mock_action_log,
                                               mock_delete_object,
                                               mock_prepare_data_name,
                                               mock_get_info,
                                               mock_get_container,
                                               mock_head_container):
        fakecontainer = 'fakecontainer'
        mock_head_container.return_value = ({'x-container-object-count': 1})
        mock_get_container.return_value = (None, [{'name': 'object1'}])
        mock_prepare_data_name.return_value = None

        resp = views.delete_container(self.request, fakecontainer, force=True)
        self.assertTrue(resp)

        kargs = mock_delete_object.mock_calls[0][2]

        self.assertEqual('fakecontainer', kargs['container'])
        self.assertEqual('object1', kargs['name'])

        self.assertTrue(mock_delete_container.called)
        self.assertTrue(mock_action_log.called)

    @patch('storage.views.main.client.head_container')
    @patch('storage.views.main.client.get_container')
    @patch('storage.views.main.get_info')
    @patch('storage.views.main.prepare_data_name')
    @patch('storage.views.main.client.delete_object')
    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.client.delete_container')
    def test_delete_container_fail_to_get_objects(self, mock_delete_container,
                                                  mock_action_log,
                                                  mock_delete_object,
                                                  mock_prepare_data_name,
                                                  mock_get_info,
                                                  mock_get_container,
                                                  mock_head_container):
        fakecontainer = 'fakecontainer'
        mock_get_container.side_effect = client.ClientException('')

        expected = False
        computed = views.delete_container(self.request, fakecontainer, True)
        self.assertEqual(computed, expected)

        self.assertFalse(mock_delete_object.called)
        self.assertFalse(mock_delete_container.called)
        self.assertFalse(mock_action_log.called)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.head_container')
    @patch('storage.views.main.get_info')
    @patch('storage.views.main.prepare_data_name')
    @patch('storage.views.main.client.get_container')
    @patch('storage.views.main.client.delete_object')
    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.client.delete_container')
    def test_delete_container_fail_to_delete_container(self,
                                                       mock_delete_container,
                                                       mock_action_log,
                                                       mock_delete_object,
                                                       mock_prepare_data_name,
                                                       mock_get_info,
                                                       mock_get_container,
                                                       mock_head_container,
                                                       mock_log_exception):
        fakecontainer = 'fakecontainer'
        mock_delete_container.side_effect = client.ClientException('')
        mock_head_container.return_value = ({'x-container-object-count': 0})
        mock_get_container.return_value = (None, None)
        mock_prepare_data_name.return_value = None

        expected = False
        computed = views.delete_container(self.request, fakecontainer, True)
        self.assertEqual(computed, expected)

        self.assertTrue(mock_log_exception.called)
        self.assertTrue(mock_delete_container.called)

    @patch('storage.views.main.client.put_object')
    def test_create_pseudofolder_with_no_prefix(self, mock_put_object):
        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({'foldername': 'fakepseudofolder'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.create_pseudofolder(self.request, project_name, 'fakecontainer')
        expected_redirect_arg = ('Location', '/p/{}/storage/objects/fakecontainer/'.format(project_name))

        self.assertIn(expected_redirect_arg, response.items())

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Pseudofolder created'))
        self.assertTrue(mock_put_object.called)

        name, args, kargs = mock_put_object.mock_calls[0]
        expected_arg = 'application/directory'

        self.assertEqual(expected_arg, kargs['content_type'])

        # Transforma em string toda a chamada do Mock, pois nao foi possivel
        #  pegar os argumentos posicionais para validar
        str_call = str(mock_put_object.call_args)

        self.assertIn(", 'fakepseudofolder/',", str_call)

    @patch('storage.views.main.client.put_object')
    def test_create_pseudofolder_with_prefix(self, mock_put_object):
        self.request.method = 'POST'
        post = self.request.POST.copy()

        prefix = 'prefix/'
        pseudofolder = 'fakepseudofolder'
        project_name = self.request.session.get('project_name')

        post.update({'foldername': pseudofolder})
        self.request.POST = post

        response = views.create_pseudofolder(self.request,
                                            project_name,
                                            'fakecontainer',
                                            prefix)

        expected_redirect_arg = (
                'Location',
                '/p/{}/storage/objects/fakecontainer/{}'.format(project_name, prefix))
        self.assertIn(expected_redirect_arg, response.items())

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Pseudofolder created'))
        self.assertTrue(mock_put_object.called)

        name, args, kargs = mock_put_object.mock_calls[0]
        expected_arg = 'application/directory'

        self.assertEqual(expected_arg, kargs['content_type'])

        # Transforma em string toda a chamada do Mock, pois nao foi possivel pegar
        # os argumentos posicionais para validar
        str_call = str(mock_put_object.call_args)

        expected_foldername = '{0}{1}/'.format(prefix, pseudofolder)
        self.assertIn(expected_foldername, str_call)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.put_object')
    def test_create_pseudofolder_exception(self, mock_put_object, mock_logging):

        mock_put_object.side_effect = client.ClientException('')

        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({'foldername': 'fakepseudofolder'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        views.create_pseudofolder(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('Access denied'))
        self.assertTrue(mock_put_object.called)
        self.assertTrue(mock_logging.called)

    @patch('storage.views.main.client.put_object')
    def test_create_pseudofolder_invalid_form(self, mock_put_object):
        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()

        post.update({'foldername': ''})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.create_pseudofolder(self.request, project_name, 'fakecontainer')

        self.assertFalse(mock_put_object.called)
        self.assertIn(_('This field is required.'), response.content.decode('UTF-8'))

    @patch('storage.views.main.delete_object')
    def test_view_delete_object_inside_a_container(self, mock_delete_object):

        mock_delete_object.return_value = True

        fakecontainer = 'fakecontainer'
        fakeobject_name = 'fakeobject'
        project_name = self.request.session.get('project_name')

        response = views.delete_object_view(self.request,
                                            project_name,
                                            fakecontainer,
                                            fakeobject_name)

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(msgs[0].message, _('Object deleted'))

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.delete_object')
    def test_view_delete_object_inside_a_pseudofolder(self, mock_delete_object):

        mock_delete_object.return_value = True

        fakecontainer = 'fakecontainer'
        fakepseudofolder = 'fakepseudofolder/'
        fakeobject_name = fakepseudofolder + 'fakeobject'
        project_name = self.request.session.get('project_name')

        response = views.delete_object_view(self.request,
                                            project_name,
                                            fakecontainer,
                                            fakeobject_name)

        msgs = [msg for msg in self.request._messages]
        self.assertEqual(msgs[0].message, _('Object deleted'))

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': fakepseudofolder,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.delete_object')
    def test_view_delete_object_fail_to_delete(self, mock_delete_object):

        mock_delete_object.return_value = False

        fakecontainer = 'fakecontainer'
        fakeobject_name = 'fakeobject'
        project_name = self.request.session.get('project_name')

        response = views.delete_object_view(self.request,
                                            project_name,
                                            fakecontainer,
                                            fakeobject_name)

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(msgs[0].message, _('Access denied'))

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.delete_object')
    def test_delete_object(self, mock_delete_object, mock_actionlog):

        fakecontainer = 'fakecontainer'
        fakeobject = 'fakeobject'

        response = views.delete_object(self.request, fakecontainer, fakeobject)

        self.assertTrue(response)
        self.assertTrue(mock_actionlog.called)

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.delete_object')
    def test_delete_object_fail_to_delete(self, mock_delete_object, mock_actionlog):

        mock_delete_object.side_effect = client.ClientException('')

        fakecontainer = 'fakecontainer'
        fakeobject = 'fakeobject'

        response = views.delete_object(self.request, fakecontainer, fakeobject)

        self.assertFalse(response)
        self.assertTrue(mock_delete_object.called)

    @patch('storage.views.main.client.delete_object')
    @patch('storage.views.main.client.get_container')
    def test_delete_empty_pseudofolder(self, mock_get_container, mock_delete_object):

        fakecontainer = 'fakecontainer'
        fakepseudofolder = 'fakepseudofolder/'
        project_name = self.request.session.get('project_name')

        mock_get_container.return_value = ['stats', [{'name': fakepseudofolder}]]

        response = views.delete_pseudofolder(self.request, project_name,
                                             fakecontainer, fakepseudofolder)

        msgs = [msg for msg in self.request._messages]

        self.assertTrue(mock_delete_object.called)
        self.assertEqual(msgs[0].message, _('Pseudofolder deleted'))

        kargs = mock_delete_object.mock_calls[0][2]

        self.assertEqual(kargs['name'], fakepseudofolder)
        self.assertEqual(kargs['container'], fakecontainer)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.client.delete_object')
    @patch('storage.views.main.client.get_container')
    def test_delete_non_empty_pseudofolder(self, mock_get_container, mock_delete_object):

        fakecontainer = 'fakecontainer'
        fakepseudofolder = 'fakepseudofolder/'
        project_name = self.request.session.get('project_name')

        fakepseudofolder_content = [
            {'name': fakepseudofolder},
            {'name': fakepseudofolder + 'fakeobjeto1'},
            {'name': fakepseudofolder + 'fakeobjeto2'}
        ]

        mock_get_container.return_value = ['stats', fakepseudofolder_content]

        views.delete_pseudofolder(self.request, project_name, fakecontainer, fakepseudofolder)

        msgs = [msg for msg in self.request._messages]
        self.assertEqual(msgs[0].message, 'Pseudofolder and 2 objects deleted.')

        self.assertTrue(mock_delete_object.called)

        kargs = mock_delete_object.mock_calls[0][2]
        self.assertEqual(kargs['name'], fakepseudofolder_content[0]['name'])

        kargs = mock_delete_object.mock_calls[1][2]
        self.assertEqual(kargs['name'], fakepseudofolder_content[1]['name'])

        kargs = mock_delete_object.mock_calls[2][2]
        self.assertEqual(kargs['name'], fakepseudofolder_content[2]['name'])

    @patch('storage.views.main.client.delete_object')
    @patch('storage.views.main.client.get_container')
    def test_delete_non_empty_pseudofolder_with_some_failures(self, mock_get_container, mock_delete_object):
        # TODO: Find a way to simulate one failures among successful deletes
        pass

    @patch('storage.views.main.client.delete_object')
    @patch('storage.views.main.client.get_container')
    def test_delete_empty_pseudofolder_inside_other_pseudofolder(self, mock_get_container, mock_delete_object):

        prefix = 'fakepseudofolder1/'
        fakecontainer = 'fakecontainer'
        fakepseudofolder = prefix + 'fakepseudofolder2/'
        project_name = self.request.session.get('project_name')

        mock_get_container.return_value = ['stats', [{'name': fakepseudofolder}]]

        response = views.delete_pseudofolder(self.request, project_name,
                                             fakecontainer, fakepseudofolder)

        msgs = [msg for msg in self.request._messages]

        self.assertTrue(mock_delete_object.called)
        self.assertEqual(msgs[0].message, _('Pseudofolder deleted'))

        kargs = mock_delete_object.mock_calls[0][2]

        self.assertEqual(kargs['name'], fakepseudofolder)
        self.assertEqual(kargs['container'], fakecontainer)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': prefix,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.client.delete_object')
    @patch('storage.views.main.client.get_container')
    def test_delete_pseudofolder_fail(self, mock_get_container, mock_delete_object):
        fakecontainer = 'fakecontainer'
        fakepseudofolder = 'fakepseudofolder/'
        project_name = self.request.session.get('project_name')

        mock_delete_object.side_effect = client.ClientException('')
        mock_get_container.return_value = ['stats', [{'name': fakepseudofolder}]]

        views.delete_pseudofolder(self.request, project_name, fakecontainer, fakepseudofolder)

        msgs = [msg for msg in self.request._messages]

        self.assertTrue(mock_delete_object.called)
        self.assertEqual(msgs[0].message, _('Fail to delete pseudofolder'))

        kargs = mock_delete_object.mock_calls[0][2]

        self.assertEqual(kargs['name'], fakepseudofolder)
        self.assertEqual(kargs['container'], fakecontainer)

    @patch('storage.views.main.client.get_account')
    def test_render_upload_view(self, mock_get_account):
        mock_get_account.return_value = fakes.get_account()
        project_name = self.request.session.get('project_name')

        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })

        response = views.upload(self.request, project_name, 'fakecontainer')

        self.assertIn('enctype="multipart/form-data"', response.content.decode('UTF-8'))

    @patch('storage.views.main.client.get_account')
    def test_render_upload_view_with_prefix(self, mock_get_account):
        mock_get_account.return_value = fakes.get_account()

        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })

        response = views.upload(self.request, 'fakecontainer', 'prefixTest')

        self.assertIn('prefixTest', response.content.decode('UTF-8'))

    @patch('storage.views.main.client.get_account')
    def test_upload_view_without_temp_key_without_prefix(self, mock_get_account):
        mock_get_account.return_value = fakes.get_account()

        patch('storage.views.main.get_temp_key',
              Mock(return_value=None)).start()

        prefix = ''
        fakecontainer = 'fakecontainer'
        project_name = self.request.session.get('project_name')

        response = views.upload(self.request, project_name, fakecontainer, prefix)

        self.assertEqual(response.status_code, 302)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': prefix,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.client.get_account')
    def test_upload_view_without_temp_key_with_prefix(self, mock_get_account):
        mock_get_account.return_value = fakes.get_account()

        patch('storage.views.main.get_temp_key',
              Mock(return_value=None)).start()

        prefix = 'prefix/'
        fakecontainer = 'fakecontainer'
        project_name = self.request.session.get('project_name')

        response = views.upload(self.request, project_name, fakecontainer, prefix)

        self.assertEqual(response.status_code, 302)

        headers = dict([i for i in response.items()])
        expected = reverse('objectview', kwargs={'container': fakecontainer,
                                                 'prefix': prefix,
                                                 'project': project_name})
        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.requests.get')
    def test_download(self, mock_get):
        content = b'ola'
        headers = {'Content-Type': 'fake/object'}
        project_name = self.request.session.get('project_name')
        mock_get.return_value = fakes.FakeRequestResponse(content=content,
                                                          headers=headers)
        response = views.download(self.request, project_name, 'fakecontainer', 'fakeobject')

        computed_headers = dict([i for i in response.items()])

        self.assertEqual(response.content, content)
        self.assertEqual(headers, computed_headers)

    @patch('storage.views.main.requests.head')
    def test_metadataview_return_headers_from_container(self, mock_head):
        headers = {'content-type': 'fake/container'}
        project_name = self.request.session.get('project_name')
        mock_head.return_value = fakes.FakeRequestResponse(content='',
                                                          headers=headers)
        response = views.metadataview(self.request, project_name, 'fakecontainer')

        self.assertIn('fake/container', response.content.decode('UTF-8'))

    @patch('storage.views.main.requests.head')
    def test_metadataview_return_headers_from_object(self, mock_head):
        headers = {'content-type': 'fake/object'}
        mock_head.return_value = fakes.FakeRequestResponse(content='',
                                                          headers=headers)
        response = views.metadataview(self.request,
                                      'fakecontainer',
                                      'fakeobject')

        self.assertIn('fake/object', response.content.decode('UTF-8'))

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.put_container')
    def test_enable_versioning(self,
                               mock_put_container,
                               mock_post_container,
                               mock_actionlog):

        container = 'fakecontainer'
        computed = views.enable_versioning(self.request, container)

        self.assertEqual(computed, True)
        self.assertTrue(mock_put_container.called)

        kargs = mock_post_container.mock_calls[0][2]

        headers = kargs['headers']
        version_location = '_version_{}'.format(container)
        self.assertEqual(version_location, headers['x-versions-location'])

        # Create container/update container
        self.assertEqual(mock_actionlog.call_count, 2)

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.put_container')
    def test_enable_versioning_fail_to_create_container(self,
                                                        mock_put_container,
                                                        mock_post_container,
                                                        mock_actionlog):

        mock_put_container.side_effect = client.ClientException('')

        container = 'fakecontainer'
        computed = views.enable_versioning(self.request, container)

        self.assertEqual(computed, False)

        self.assertFalse(mock_post_container.called)

        self.assertEqual(mock_actionlog.call_count, 0)

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.put_container')
    def test_enable_versioning_fail_to_update_container(self,
                                                        mock_put_container,
                                                        mock_post_container,
                                                        mock_actionlog):

        mock_post_container.side_effect = client.ClientException('')

        container = 'fakecontainer'
        computed = views.enable_versioning(self.request, container)

        self.assertEqual(computed, False)

        self.assertTrue(mock_post_container.called)

        self.assertEqual(mock_actionlog.call_count, 1)

    @patch('storage.views.main.actionlog.log')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.delete_container')
    @patch('storage.views.main.client.head_container')
    def test_disable_versioning(self,
                                mock_head_container,
                                mock_delete_container,
                                mock_post_container,
                                mock_actionlog):

        version_location = '_version_fakecontainer'
        mock_head_container.return_value = {'x-versions-location': version_location}

        container = 'fakecontainer'
        computed = views.disable_versioning(self.request, container)

        self.assertEqual(computed, True)

        kargs = mock_delete_container.mock_calls[0][2]

        self.assertEqual(version_location, kargs['container'])

        self.assertEqual(mock_actionlog.call_count, 1)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.delete_container')
    @patch('storage.views.main.client.head_container')
    def test_disable_versioning_fail_to_get_container_headers(self,
                                                       mock_head_container,
                                                       mock_delete_container,
                                                       mock_post_container,
                                                       mock_logging):

        mock_head_container.side_effect = client.ClientException('')

        container = 'fakecontainer'
        computed = views.disable_versioning(self.request, container)

        self.assertEqual(computed, False)

        msgs = [msg for msg in self.request._messages]
        self.assertEqual(msgs[0].message, _('Access denied'))

        self.assertEqual(mock_logging.call_count, 1)

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.delete_container')
    @patch('storage.views.main.client.head_container')
    def test_disable_versioning_fail_to_update_container_header(self,
                                                       mock_head_container,
                                                       mock_delete_container,
                                                       mock_post_container,
                                                       mock_logging):

        mock_post_container.side_effect = client.ClientException('')

        version_location = '_version_fakecontainer'
        mock_head_container.return_value = {'x-versions-location': version_location}

        container = 'fakecontainer'
        computed = views.disable_versioning(self.request, container)

        self.assertEqual(computed, False)

        self.assertFalse(mock_delete_container.called)

        msgs = [msg for msg in self.request._messages]
        self.assertEqual(msgs[0].message, _('Access denied'))

        self.assertEqual(mock_logging.call_count, 1)

    @patch('storage.views.main.render')
    @patch('storage.views.main.client.head_container')
    def test_object_versioning_view_versioning_disabled(self,
                                                        mock_head_container,
                                                        mock_render):
        mock_head_container.return_value = {}
        project_name = self.request.session.get('project_name')

        views.object_versioning(self.request, project_name, 'fakecontainer')

        kargs = mock_render.mock_calls[0][1]
        computed = kargs[2]

        expected = {'objects': utils.generic_pagination([], 1),
                    'container': 'fakecontainer',
                    'version_location': None}

        self.assertEqual(str(computed['objects']), str(expected['objects']))
        self.assertEqual(computed['container'], expected['container'])
        self.assertEqual(computed['version_location'], expected['version_location'])

    @patch('storage.views.main.render')
    @patch('storage.views.main.client.get_container')
    @patch('storage.views.main.client.head_container')
    def test_object_versioning_view_versioning_enabled(self,
                                                       mock_head_container,
                                                       mock_get_container,
                                                       mock_render):
        mock_head_container.return_value = {'x-versions-location': 'abc'}
        mock_get_container.return_value = (None, [])
        project_name = self.request.session.get('project_name')

        views.object_versioning(self.request, project_name, 'fakecontainer')

        kargs = mock_render.mock_calls[0][1]
        computed = kargs[2]

        expected = {'objects': utils.generic_pagination([], 1),
                    'container': 'fakecontainer',
                    'version_location': 'abc'}

        self.assertEqual(str(computed['objects']), str(expected['objects']))
        self.assertEqual(computed['container'], expected['container'])
        self.assertEqual(computed['version_location'], expected['version_location'])

    @patch('storage.views.main.enable_versioning')
    def test_object_versioning_view_enabling_versioning(self, mock_enable):

        post = self.request.method = 'POST'
        post = self.request.POST.copy()
        project_name = self.request.session.get('project_name')

        post.update({'action': 'enable'})
        self.request.POST = post

        response = views.object_versioning(self.request, project_name, 'fakecontainer')

        self.assertEqual(mock_enable.call_count, 1)

        headers = dict([i for i in response.items()])
        expected = reverse('object_versioning',
                           kwargs={'project': project_name, 'container': 'fakecontainer'})

        self.assertEqual(headers['Location'], expected)

    @patch('storage.views.main.disable_versioning')
    def test_object_versioning_view_disabling_versioning(self, mock_disable):

        post = self.request.method = 'POST'
        post = self.request.POST.copy()
        project_name = self.request.session.get('project_name')

        post.update({'action': 'disable'})
        self.request.POST = post

        response = views.object_versioning(self.request, project_name, 'fakecontainer')

        self.assertEqual(mock_disable.call_count, 1)

        headers = dict([i for i in response.items()])
        expected = reverse('object_versioning',
                           kwargs={'project': project_name, 'container': 'fakecontainer'})

        self.assertEqual(headers['Location'], expected)

    @override_settings(SWIFT_HIDE_PREFIXES=['.'])
    @patch('storage.views.main.client.get_account')
    def test_filter_containers_with_prefix_listed_in_SWIFT_HIDE_PREFIXES(self, mock_get_account):
        fake_get_acc = fakes.get_account()
        containers = [{'count': 4, 'bytes': 4, 'name': '.container4'}]
        project_name = self.request.session.get('project_name')

        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })

        mock_get_account.return_value = (fake_get_acc[0], containers)
        response = views.containerview(self.request, project_name)

        self.assertEqual(response.status_code, 200)
        self.assertNotIn('/storage/objects/.container4/', response.content.decode('UTF-8'))

    @patch('json.loads')
    @patch('requests.get')
    def test_get_info(self, mock_get, mock_json_loads):
        expect_url = 'https://fake.api.globoi.com/info'
        url = "https://fake.api.globoi.com/v1/AUTH_12314"

        views.main.get_info(url)
        mock_json_loads.assert_called()
        mock_get.assert_called_with(expect_url)

    def test_prepare_data_name(self):
        fakecontainer = 'fakecontainer'
        fake_obj_name = 'fakename'

        expectd_data_name = fakecontainer.encode() + b"/" + fake_obj_name.encode()
        data_name = views.main.prepare_data_name(fakecontainer, fake_obj_name)

        self.assertEqual(data_name, expectd_data_name)

    @patch('storage.views.main.client.head_container')
    @patch('storage.views.main.client.get_container')
    @patch('storage.views.main.get_info')
    @patch('storage.views.main.prepare_data_name')
    @patch('requests.post')
    @patch("storage.views.main.actionlog.log")
    @patch('storage.views.main.client.delete_container')
    def test_delete_container_bulk_delete(self, mock_delete_container,
                                          mock_action_log,
                                          mock_post,
                                          mock_prepare_data_name,
                                          mock_get_info,
                                          mock_get_container,
                                          mock_head_container):
        fakecontainer = 'fakecontainer'
        fake_obj_name = 'fakename'
        headers = {"X-Auth-Token": None}
        expected_data = fakecontainer.encode() + b"/" + fake_obj_name.encode()
        mock_head_container.return_value = ({'x-container-object-count': 1})
        mock_get_container.return_value = (None, [{'name': 'object1'}])
        mock_get_info.return_value = {'bulk_delete': {'max_deletes_per_request': 1}}
        mock_prepare_data_name.return_value = expected_data

        resp = views.delete_container(self.request, fakecontainer, force=True)
        self.assertTrue(resp)
        mock_head_container.assert_called()
        mock_get_container.assert_called()
        mock_get_info.assert_called()
        mock_prepare_data_name.assert_called()
        mock_post.assert_called_with('https://fake.api.globoi.com/v1/AUTH_1?bulk-delete=true', headers=headers,
                                     data=expected_data)
        mock_delete_container.assert_called()
        mock_action_log.assert_called()


class TestStorageAcls(BaseTestCase):

    def test_edit_acl_needs_authentication(self):
        response = views.edit_acl(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    @patch('storage.views.main.client.head_container')
    def test_edit_acl_list_acls_container_public(self, mock_get_container):
        """
        Verify the ACL list for a container public and
        if the "Make private" action is available
        """
        mock_get_container.return_value = {
            'x-container-read': '.r:*',
            'x-container-write': '',
        }
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertIn('Add ACL to container fakecontainer', response.content.decode('UTF-8'))
        self.assertIn('Make private', response.content.decode('UTF-8'))
        self.assertIn('Add ACL', response.content.decode('UTF-8'))
        self.assertIn('.r:*', response.content.decode('UTF-8'))

    @patch('storage.views.main.client.head_container')
    def test_edit_acl_list_acls_container_private(self, mock_get_container):
        """
        Verify the ACL list for a private container with no ACLS and
        if the "Make Public" action is available
        """
        mock_get_container.return_value = {
            'x-container-read': '',
            'x-container-write': '',
        }
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertIn('Make public', response.content.decode('UTF-8'))
        self.assertIn('Add ACL', response.content.decode('UTF-8'))

        expected = 'There are no ACLs for this container yet. Add a new ACL by clicking the red button.'
        self.assertIn(expected, response.content.decode('UTF-8'))

    @patch('storage.views.main.client.head_container')
    def test_edit_acl_private_container_but_public_for_read_and_write_for_an_user_and_project(self, mock_head_container):
        """
        Verify if it's properly listing container's acl and
        if the "Make Public" action is available
        """
        mock_head_container.return_value = {
            'x-container-read': 'projectfake:userfake',
            'x-container-write': 'projectfake:userfake',
        }
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertIn('Make public', response.content.decode('UTF-8'))
        self.assertIn('Add ACL', response.content.decode('UTF-8'))
        self.assertIn('projectfake:userfake', response.content.decode('UTF-8'))

    @patch('storage.views.main.client.head_container')
    def test_edit_acl_public_container_and_public_for_read_for_more_than_one_user_and_project(self, mock_head_container):
        """
        Verify if it's properly listing container's acl for a public container
        and if Make Private is available
        """
        mock_head_container.return_value = {
            'x-container-read': '.r:*,projectfake:userfake',
            'x-container-write': 'projectfake2:userfake2',
        }
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertIn('Make private', response.content.decode('UTF-8'))
        self.assertIn('Add ACL', response.content.decode('UTF-8'))
        self.assertIn('projectfake:userfake', response.content.decode('UTF-8'))
        self.assertIn('projectfake2:userfake2', response.content.decode('UTF-8'))

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_grant_read_and_write_permission_for_a_project_and_user(self, mock_head_container, mock_post_container):
        mock_head_container.return_value = {
            'x-container-read': '',
            'x-container-write': '',
        }

        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({
                'username': 'projectfake:userfake',
                'read': 'On',
                'write': 'On'
        })

        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {
            'X-Container-Write': ',projectfake:userfake',
            'X-Container-Read': ',projectfake:userfake'
        }
        self.assertEqual(expected_arg, kargs['headers'])

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].message, _('ACLs updated'))

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_expcetion_on_grant_read_and_write_permission_for_a_project_and_user(self, mock_head_container, mock_post_container, mock_logging):
        mock_head_container.return_value = {
            'x-container-read': '',
            'x-container-write': '',
        }

        mock_post_container.side_effect = client.ClientException('')

        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({
                'username': 'projectfake:userfake',
                'read': 'On',
                'write': 'On'
        })

        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(msgs), 1)
        self.assertTrue(mock_logging.called)
        self.assertEqual(msgs[0].message, _('ACL update failed'))

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_make_private(self, mock_head_container, mock_post_container):
        """
            Verify if the action "Making Private" is
            removing ".r:*" from x-container-read header
        """
        mock_head_container.return_value = {
            'x-container-read': '.r:*,projectfake:userfake',
            'x-container-write': 'projectfake2:userfake2',
        }

        self.request.method = 'GET'
        get = self.request.GET.copy()

        get.update({'delete': '.r:*,.rlistings'})
        self.request.GET = get
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertTrue(mock_post_container.called)

        name, args, kargs = mock_post_container.mock_calls[0]
        expected_arg = {
            'X-Container-Write': 'projectfake2:userfake2,',
            'X-Container-Read': 'projectfake:userfake,'
        }
        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_make_public(self, mock_head_container, mock_post_container):
        """
            Verify if the action "Making Public" is
            including ".r:*" in x-container-read header
        """
        mock_head_container.return_value = {
            'x-container-read': 'projectfake:userfake',
            'x-container-write': 'projectfake2:userfake2',
        }

        self.request.method = 'POST'
        post = self.request.POST.copy()

        post.update({'username': '.r:*', 'read': 'On'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)
        self.assertTrue(mock_post_container.called)

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {
            'X-Container-Write': 'projectfake2:userfake2',
            'X-Container-Read': 'projectfake:userfake,.r:*'
        }
        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_delete_acl_for_user_in_a_public_container(self, mock_head_container, mock_post_container):
        """ Verify if is deleting the correct ACL """
        mock_head_container.return_value = {
            'x-container-read': '.r:*,projectfake:userfake',
            'x-container-write': 'projectfake:userfake,projectfake2:userfake2',
        }

        self.request.method = 'GET'
        get = self.request.GET.copy()

        get.update({'delete': 'projectfake:userfake'})
        self.request.GET = get
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {
            'X-Container-Write': 'projectfake2:userfake2,',
            'X-Container-Read': '.r:*,'
        }
        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_delete_acl_for_user_in_a_private_container(self, mock_head_container, mock_post_container):
        mock_head_container.return_value = {
            'x-container-read': 'projectfake:userfake',
            'x-container-write': 'projectfake:userfake,projectfake2:userfake2',
        }

        self.request.method = 'GET'
        get = self.request.GET.copy()

        get.update({'delete': 'projectfake:userfake'})
        self.request.GET = get
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        self.assertEqual(response.status_code, 200)

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {
            'X-Container-Write': 'projectfake2:userfake2,',
            'X-Container-Read': ''
        }
        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.log.exception')
    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_edit_acl_delete_acl_exception(self, mock_head_container, mock_post_container, mock_logging):
        mock_head_container.return_value = {
            'x-container-read': 'projectfake:userfake',
            'x-container-write': 'projectfake:userfake',
        }

        mock_post_container.side_effect = client.ClientException('')

        self.request.method = 'GET'
        get = self.request.GET.copy()

        get.update({'delete': 'projectfake:userfake'})
        self.request.GET = get
        project_name = self.request.session.get('project_name')

        response = views.edit_acl(self.request, project_name, 'fakecontainer')

        msgs = [msg for msg in self.request._messages]

        self.assertEqual(len(msgs), 1)
        self.assertTrue(mock_logging.called)
        self.assertEqual(msgs[0].message, _('ACL update failed'))
        self.assertIn('projectfake:userfake', response.content.decode('UTF-8'))


class TestStorageCORS(BaseTestCase):

    def test_edit_cors_needs_authentication(self):
        response = views.edit_cors(self.anonymous_request)

        self.assertEqual(response.status_code, 302)

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_define_novo_host_para_regra_de_cors_no_container(self, mock_head_container, mock_post_container):
        mock_head_container.return_value = {
            'x-container-meta-access-control-allow-origin': '',
        }

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()
        project_name = self.request.session.get('project_name')

        post.update({'host': 'globo.com'})
        self.request.POST = post

        views.edit_cors(self.request, project_name, 'fakecontainer')

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {'x-container-meta-access-control-allow-origin': 'globo.com'}

        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.client.post_container')
    @patch('storage.views.main.client.head_container')
    def test_remove_host_da_regra_de_cors_do_container(self, mock_head_container, mock_post_container):
        mock_head_container.return_value = {
            'x-container-meta-access-control-allow-origin': 'globo.com globoi.com',
        }

        self.request.method = 'GET'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        get = self.request.GET.copy()
        project_name = self.request.session.get('project_name')

        get.update({'delete': 'globo.com'})
        self.request.GET = get

        views.edit_cors(self.request, project_name, 'fakecontainer')

        name, args, kargs = mock_post_container.mock_calls[0]

        expected_arg = {'x-container-meta-access-control-allow-origin': 'globoi.com'}

        self.assertEqual(expected_arg, kargs['headers'])

    @patch('storage.views.main.requests.post')
    def test_remove_from_cache_status_201(self, mock_requests_post):
        mock_requests_post.return_value = fakes.FakeRequestResponse(201)

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()

        post.update({'urls': 'http://localhost/1\r\nhttp://localhost/2'})
        self.request.POST = post
        project_name = self.request.session.get('project_name')

        response = views.remove_from_cache(self.request, project_name)

        host, kargs = mock_requests_post.call_args

        expected_json = {
            'json': {
                'url': [u'http://localhost/1', u'http://localhost/2'],
                'user': self.request.user.username
            }
        }

        # Verifica se a chamada para a API estava com os argumentos corretos
        self.assertEqual(expected_json, kargs)



# -*- coding: utf-8 -*-

from allaccess.clients import OAuth2Client
from requests.structures import CaseInsensitiveDict


class BearerOAuth2(object):

    def __init__(self, user_token):
        self.user_token = user_token

    def __call__(self, r):
        r.headers['Authorization'] = 'Bearer {0}'.format(self.user_token)
        return r


class OAuth2BearerClient(OAuth2Client):

    def request(self, method, url, **kwargs):
        user_token = kwargs.pop('token', self.token)
        token, _ = self.parse_raw_token(user_token)

        headers = CaseInsensitiveDict(kwargs.get('headers', {}))
        kwargs['headers'] = self._apply_json_as_accept_type_when_not_defined(
            headers)

        if token is not None:
            kwargs['auth'] = BearerOAuth2(user_token=token)

        return super(OAuth2BearerClient, self).request(method, url, **kwargs)

    def _apply_json_as_accept_type_when_not_defined(self, headers):
        defined_accept = headers.get('Accept')
        if not defined_accept:
            headers['Accept'] = 'application/json'
        return headers



# -*- coding: utf-8 -*-

import json

from unittest.mock import patch, Mock
from unittest import TestCase

from swiftclient import client

from storage.tests import fakes
from storage import views

from vault.tests.fakes import fake_request


class TestSwiftTrash(TestCase):

    def setUp(self):
        self.request = fake_request()

        # silence log
        patch('storage.views.log',
              Mock(return_value=None)).start()

        patch('identity.keystone.Keystone',
              Mock(return_value=None)).start()

    def tearDown(self):
        patch.stopall()

    def test_deleted_objects_list_needs_authentication(self):
        req = fake_request(user=False)
        response = views.get_deleted_objects(req)

        self.assertEqual(response.status_code, 302)

    @patch("storage.views.client.get_container")
    def test_get_deleted_objects_returns_a_json(self, mock_get_container):
        mock_get_container.return_value = fakes.get_container()
        project_name = self.request.session.get('project_name')

        response = views.get_deleted_objects(self.request, project_name, 'container1')

        self.assertIn(b'application/json', response.serialize_headers())

    @patch("storage.views.client.get_container")
    def test_check_properties_from_get_deleted_objects_content(self, mock_get_container):
        mock_get_container.return_value = fakes.get_container(trash_container=".trash-container1")
        project_name = self.request.session.get('project_name')

        response = views.get_deleted_objects(self.request, project_name, 'container1')
        expected_items = ["deleted_objects", "prefix", "storage_url",
                          "trash_container", "original_container"]

        result = json.loads(response.content)
        for item in expected_items:
            self.assertIn(item, result)

    @patch("storage.views.client.get_container")
    def test_get_deleted_objects_client_exception(self, mock_get_container):
        mock_get_container.side_effect = client.ClientException("error message",
                                                                http_status=500)
        project_name = self.request.session.get('project_name')

        response = views.get_deleted_objects(self.request, project_name, 'container1')

        self.assertEqual(response.content.decode(), '{"error": "error message"}')
        self.assertEqual(response.status_code, 500)

    @patch("storage.views.remove_from_trash")
    def test_deleted_objects_from_trash(self, mock_trash_remove):
        mock_trash_remove.return_value = fakes.FakeRequestResponse(200)

        response = views.remove_from_trash(self.request)

        self.assertEqual(response.status_code, 200)

    @patch("storage.views.remove_from_trash")
    def test_deleted_objects_from_trash_with_fail(self, mock_trash_remove):
        mock_trash_remove.return_value = fakes.FakeRequestResponse(500)

        response = views.remove_from_trash(self.request)

        self.assertEqual(response.status_code, 500)

    @patch("storage.views.delete_pseudofolder")
    def test_deleted_objects_from_pseudofolder(self, mock_remove_from_pseudo_folder):
        mock_remove_from_pseudo_folder.return_value = fakes.FakeRequestResponse(200)

        response = views.delete_pseudofolder(self.request, "container1", "pseudofolder")

        self.assertEqual(response.status_code, 200)

    @patch("storage.views.delete_pseudofolder")
    def test_deleted_objects_from_pseudofolder_with_fail(self, mock_remove_from_pseudo_folder):
        mock_remove_from_pseudo_folder.return_value = fakes.FakeRequestResponse(500)

        response = views.delete_pseudofolder(self.request, "container1", "pseudofolder")

        self.assertEqual(response.status_code, 500)

    @patch("storage.views.restore_object")
    def test_restore_object_from_trash(self, mock_restore):
        mock_restore.return_value = fakes.FakeRequestResponse(202)

        response = views.restore_object(self.request)

        self.assertEqual(response.status_code, 202)

    @patch("storage.views.restore_object")
    def test_restore_object_from_trash_with_fail_error_on_put_object(self, mock_restore):

        mock_restore.return_value = fakes.FakeRequestResponse(500)

        response = views.restore_object(self.request)

        self.assertEqual(response.status_code, 500)

    @patch("storage.views.restore_object")
    def test_restore_object_from_trash_with_fail_object_exists_on_container(self, mock_restore):

        mock_restore.return_value = fakes.FakeRequestResponse(409)

        response = views.restore_object(self.request)

        self.assertEqual(response.status_code, 409)



# -*- coding: utf-8 -*-

import random
import string
import logging
import requests

from keystoneclient import exceptions, v3

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import User, Group

from vault.models import GroupProjects
from identity.models import Project


log = logging.getLogger(__name__)


class KeystoneBase:

    def __init__(self, username=None, password=None, project_name=None,
                 auth_url=None, config={}):

        self.config = {
            'version': 3,
            'timeout': settings.KEYSTONE_TIMEOUT,

            'auth_url': auth_url or settings.KEYSTONE_URL,
            'username': username or settings.KEYSTONE_USERNAME,
            'password': password or settings.KEYSTONE_PASSWORD,
            'project_name': project_name or settings.KEYSTONE_PROJECT,
        }

        self.config.update(config)

        # keystone connection
        self.conn = None
        if self.allow_connection():
            self.conn = self._create_keystone_connection()

    def allow_connection(self):
        return True

    def _create_keystone_connection(self):
        client = v3.client

        return client.Client(**self.config)

    def vault_database_project_delete(self, project_id):
        project = Project.objects.filter(project=project_id)
        project.delete()

    def vault_group_project_delete(self, project_id):
        group_projects = GroupProjects.objects.filter(project=project_id)
        group_projects.delete()

    def vault_group_project_create(self, group_id, project_id, owner=0):
        group_project = GroupProjects(group_id=group_id,
                                      project=project_id,
                                      owner=owner)
        group_project.save()
        return group_project

    def vault_user_get(self, user_id):
        return User.objects.get(id=user_id)

    def vault_user_get_by_name(self, username):
        return User.objects.get(username=username)

    def vault_user_list(self):
        return User.objects.all()

    def vault_user_group_get(self, user_id, group_id):
        return User.objects.get(id=user_id, groups__id=group_id)

    def vault_user_group_list(self, user_id):
        return Group.objects.filter(user__id=user_id)

    def vault_group_get_by_name(self, name):
        return Group.objects.get(name=name)

    # based on: https://github.com/openstack/horizon/blob/master/openstack_dashboard/api/keystone.py#L51-L56
    def _project_manager(self):
        return self.conn.projects

    # based on: https://github.com/openstack/horizon/blob/master/openstack_dashboard/api/keystone.py#L45-L49
    def _user_manager(self, user):
        if getattr(user, "default_project_id", None) is None:
            user.project_id = getattr(user, "tenantId", None)
        return user

    def user_list(self, project_id=None):
        """
        List project users. If no project is defined, it will list all users.

        :param project_id: string or None
        :return: list
        :raises: NotFound if project does not exists
        """
        return self.conn.users.list(project_id)

    def user_get(self, user_id):
        user = self.conn.users.get(user_id)
        return self._user_manager(user)

    def user_create(self, name, email=None, password=None, enabled=True,
                    domain=None, project_id=None, role_id=None):

        user = self.conn.users.create(name, password=password, email=email,
                                      project=project_id, enabled=enabled,
                                      domain=domain)

        # Assign role and project to user
        if project_id is not None and role_id is not None:
            role = self.role_get(role_id)
            project = self.project_get(project_id)

            # V2 the '_member_' role is automatically linked
            # if self.config['version'] > 2 or role.name != '_member_':
            self.add_user_role(user, project, role)

        return user

    def user_update(self, user, **data):
        # If password is /False/, removes it to not update with /False/
        if not data['password']:
            data.pop('password')

        user = self.conn.users.update(user, **data)

        return user

    def user_update_password(self, user, password):
        return self.conn.users.update(user, password=password)

    def user_delete(self, user_id):
        return self.conn.users.delete(user_id)

    def user_get_by_name(self, user_name):
        users = self.user_list()

        for user in users:
            if user.name == user_name:
                return user

        return None

    def user_update_enabled(self, user_id, enabled=False):
        return self.conn.users.update_enabled(user_id, enabled)

    def project_create(self, name, domain_id='default', description=None,
                       enabled=True, **kwargs):
        conn = self._project_manager()

        return conn.create(name, domain_id, description=description,
                           enabled=enabled, **kwargs)

    def project_update(self, project, **kwargs):
        conn = self._project_manager()

        return conn.update(project, **kwargs)

    def project_delete(self, project_id):
        conn = self._project_manager()
        return conn.delete(project_id)

    def project_update_enabled(self, project_id, enabled=False):
        project = self.project_get(project_id)
        return project.update(enabled=enabled)

    def project_list(self):
        conn = self._project_manager()
        return conn.list()

    def project_get(self, project_id):
        conn = self._project_manager()
        return conn.get(project_id)

    def project_get_by_name(self, project_name):
        projects = self.project_list()

        for project in projects:
            if project.name == project_name:
                return project

        return None

    def role_list(self):
        return self.conn.roles.list()

    def role_get(self, role_id):
        return self.conn.roles.get(role_id)

    def role_find(self, name):
        return self.conn.roles.find(name=name)

    def list_user_roles(self, user=None, project=None):
        roleManager = v3.roles.RoleManager(self.conn)
        return roleManager.list(user=user, project=project)

    def add_user_role(self, user=None, project=None, role=None):
        return self.conn.roles.grant(role, user=user, project=project)

    def remove_user_role(self, user=None, project=None, role=None):
        return self.conn.roles.revoke(role, user=user, project=project)

    def role_assignments_list(self, user=None, group=None, project=None, domain=None, role=None):
        return self.conn.role_assignments.list(
            user=user, group=group, project=project, domain=domain, role=role)

    def vault_project_create(self, project_name, group_id, description=None,
                             **kwargs):
        """
        This method do the complete project create process:
            - Creates project
            - creates user
            - add swiftoperator role
            - link the project to a team
        """

        # Creates project
        try:
            project = self.project_create(project_name,
                                          description=description,
                                          enabled=True,
                                          **kwargs)
        except exceptions.Conflict as err:
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Duplicated project name.'
            }
        except exceptions.Forbidden as err:
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Superuser required.'
            }

        swiftop = settings.KEYSTONE_ROLE or \
                  self.conn.roles.find(name='swiftoperator').id
        # Creates admin user and add swiftoperator role
        try:
            admin_password = Keystone.create_password()
            admin_user = self.user_create(
                name='u_{}'.format(project_name),
                password=admin_password,
                role_id=swiftop,
                project_id=project.id
            )
        except exceptions.Forbidden as err:
            self.project_delete(project.id)
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Admin User required'
            }

        # Creates internal user and add swiftoperator role
        try:
            user_name = 'u_vault_{}'.format(project_name)
            internal_password = Keystone.create_password()
            internal_user = self.user_create(
                name=user_name,
                email=self.request.user.email,
                password=internal_password,
                project_id=project.id,
                enabled=project.enabled,
                domain=project.domain_id,
                role_id=swiftop
            )
        except exceptions.Forbidden as err:
            self.user_delete(admin_user.id)
            self.project_delete(project.id)
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Internal User required'
            }

        # Creates project user
        try:
            from vault.utils import encrypt_password
            password = encrypt_password(internal_password)
            db_project = Project(
                project=project.id,
                user=user_name,
                password=password.decode("utf-8")
            )
            db_project.save()
        except Exception as err:
            self.user_delete(admin_user.id)
            self.user_delete(internal_user.id)
            self.project_delete(project.id)
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Project User required'
            }

        # Link the project to a team
        try:
            self.vault_group_project_create(group_id=group_id,
                                            project_id=project.id,
                                            owner=1)
        except Exception as err:
            self.user_delete(admin_user.id)
            self.user_delete(internal_user.id)
            self.project_delete(project.id)
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': 'Unable to assign project to group'
            }

        return {
            'status': True,
            'project': project,
            'user': admin_user,
            'password': admin_password
        }

    def vault_set_project_owner(self, project_id, group_id):
        current = (GroupProjects.objects.filter(project=project_id, owner=1)
                                        .first())

        if current and int(current.group_id) == int(group_id):
            return {
                'status': False,
                'reason': 'Group already owner'
            }

        if current:
            current.owner = 0

        new = (GroupProjects.objects.filter(project=project_id,
                                            group_id=group_id,
                                            owner=0)
                                    .first())
        if new:
            new.owner = 1
        else:
            self.vault_group_project_create(group_id, project_id, 1)

        current.save()
        new.save()

        return {
            'status': True
        }

    def vault_project_update(self, project_id, project_name, group_id,
                             **kwargs):
        project = self.project_get(project_id)
        try:
            updated_project = self.project_update(project,
                                                  name=project_name,
                                                  **kwargs)
        except exceptions.Forbidden:
            return {
                'status': False,
                'reason': 'Admin required'
            }

        if group_id is not None:
            self.vault_set_project_owner(project_id, group_id)

        return {
            'status': True,
            'project': updated_project
        }

    def vault_project_delete(self, project_name):
        """
        Delete project and the "u_<project_name>" user.
        """

        project = self.project_get_by_name(project_name)
        if not project:
            return {
                'status': False,
                'reason': 'Project not found'
            }

        # This is the code that disables project and user
        # intead of delete them
        # try:
        #     project = self.project_update_enabled(project.id, False)
        # except Exception as err:
        #     return {
        #         'status': False,
        #         'reason': err
        #     }

        # user = self.find_user_with_u_prefix(project.id, 'u')
        # if user:
        #     self.user_update_enabled(user.id, False)

        # Delete from Swift
        del_swift = self._project_delete_swift(project.id)

        # Swift returns 404 if the project had never been used
        if del_swift.status_code not in (204, 404):
            reason = 'Unable to delete project from Swift (HTTP {})'.format(
                del_swift.status_code)

            return {
                'status': False,
                'reason': reason
            }

        # Delete user named "u_<project_name>"
        user = self.find_user_with_u_prefix(project.id, 'u')
        if user:
            self.user_delete(user.id)

        # Delete user named "u_vault_<project_name>"
        user = self.find_user_with_u_prefix(project.id, 'u_vault')
        if user:
            self.user_delete(user.id)

        # Delete project from vault database
        self.vault_database_project_delete(project.id)

        # Delete group_project
        self.vault_group_project_delete(project.id)

        # Delete from Keystone
        try:
            self.project_delete(project.id)
        except Exception as err:
            log.error('Error: {}'.format(err))
            return {
                'status': False,
                'reason': err
            }

        return {
            'status': True
        }

    def _project_delete_swift(self, project_id):

        headers = {
            'X-Storage-Token': self.conn.auth_token
        }
        verify = not self.config.get('insecure')

        # replace project_id in url to the id of the project that will
        # be deleted
        storage_url = self.get_object_store_endpoints()['adminURL']
        _, project_admin_id = storage_url.split('AUTH_')
        url = storage_url.replace(project_admin_id, project_id)

        return requests.delete(url, headers=headers, verify=verify)

    def find_user_with_u_prefix(self, project_id, prefix):
        project = self.project_get(project_id)
        users = self.user_list(project.id)

        for user in users:
            if user.name == '{}_{}'.format(prefix, project.name):
                return user

        return None

    @staticmethod
    def create_password():
        caracteres = string.ascii_letters + string.digits + '!@#$%&*_'
        return ''.join(random.choice(caracteres) for x in range(12))

    def get_endpoints(self):
        sc_endpoints = self.conn.service_catalog.get_endpoints()
        endpoints = {}

        for service, item in sc_endpoints.items():
            service_name = service.replace('-', '_')
            endpoints[service_name] = {
                a['interface'] + 'URL': a['url'] for a in item
            }
        return endpoints

    def get_object_store_endpoints(self):
        endpoints = self.get_endpoints()
        return endpoints.get('object_store')


class KeystoneNoRequest(KeystoneBase):

    def __init__(self, username=None, password=None, project_name=None,
                 auth_url=None):
        return super(KeystoneNoRequest, self).__init__(
            username, password, project_name, auth_url)


class Keystone(KeystoneBase):

    def __init__(self, request=None, username=None, password=None,
                 project_name=None, auth_url=None):

        self.request = request
        extra_config = {
            'remote_addr': self.request.environ.get('REMOTE_ADDR', ''),
            'insecure': False,
        }

        super(Keystone, self).__init__(username, password, project_name,
                                       auth_url, config=extra_config)

    def allow_connection(self):
        return self._is_allowed_to_connect()

    def _is_allowed_to_connect(self):
        """
        Check if logged user can access the project set on session.
        If no project was set, it means that it should connect to the "admin"
        project.
        """

        project_id = self.request.session.get('project_id')
        groups = self.request.user.groups.all()
        group_projects = []

        if project_id:
            group_projects = GroupProjects.objects.filter(group__in=groups,
                                                          project=project_id)
        else:
            # caso nao exista project_id no ambiente, preciso
            # verificar o caso de um usuario num grupo recem-criado
            # sem projeto definido
            group_projects = GroupProjects.objects.filter(group__in=groups)
            if not group_projects and not project_id:
                return True

        # Pode autenticar se project pertence ao time do usuario
        if not group_projects and not self.request.user.is_superuser:
            log.warning(_('Permission denied to manage this project'))
            return False

        return True



# -*- coding: utf-8 -*-

from unittest.mock import patch, Mock
from unittest import TestCase

from actionlogger.actionlogger import ActionLogger, ActionNotFound


class TestActionLogger(TestCase):

    def setUp(self):
        self.mock_audit = patch('actionlogger.actionlogger.Audit',
                                retun=Mock()).start()

        self.action_logger = ActionLogger()

    def tearDown(self):
        patch.stopall()

    def test_make_message(self):
        computed = self.action_logger._make_log_message(
            'User1', 'create', 'A Test Project')
        expected = 'Usuario User1 Criou A Test Project'

        self.assertEqual(computed, expected)

        computed = self.action_logger._make_log_message(
            'User2', 'update', 'A Test Project')
        expected = 'Usuario User2 Atualizou A Test Project'

        self.assertEqual(computed, expected)

    @patch('actionlogger.actionlogger.syslog.syslog')
    def test_log_invoke_syslog_info_level(self, mock):
        self.action_logger.log('User1', 'create', 'A Test Project')
        self.assertEqual(1, mock.call_count)
        mock.assert_called_with(6, 'Usuario User1 Criou A Test Project')

    @patch('actionlogger.actionlogger.Audit')
    def test_log_set_and_invoke_audit_model(self, audit_mock):
        save_mock = audit_mock.return_value.save
        save_mock.return_value = True
        self.action_logger.log('User1', 'create', 'A Test Project')
        self.assertEqual(1, save_mock.call_count)

    def test_log_with_an_invalid_action_raises_actionnotfound(self):
        self.assertRaises(ActionNotFound, self.action_logger.log,
                          'TestUser', 'drop', 'A Test Project')



# -*- coding: utf-8 -*-

from django import template

register = template.Library()


@register.inclusion_tag('vault/pagination.html', takes_context=True)
def pagination(context, paginated_item):
    return {
        'items': paginated_item,
        'get_parameters': dict(context['request'].GET)
    }



# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

VERSION = __import__('vault').__version__

setup(
    name='vault',
    version=VERSION,
    description='Admin webapp for OpenStack Keystone and OpenStack Swift.',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    author='Storm',
    author_email='storm@g.globo',
    url='https://github.com/globocom/vault',
    install_requires=[
        'Babel==2.3.4',
        'gunicorn==19.3.0',
        'Django==3.1.6',
        'django3-all-access==0.10.0',
        'iso8601==0.1.11',
        'mysqlclient==1.4.6',
        'netaddr==0.7.18',
        'oslo.config==5.2.0',
        'python-dateutil==2.5.3',
        'python-keystoneclient==3.16.0',
        'python-swiftclient==3.7.1',
        'pytz==2015.4',
        'requests==2.19.1',
        'cryptography==2.9.2'
    ],
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
)



# pylint:disable=E1101

""" Standalone webinterface for Openstack Swift. """
import math
import os
import time
import json
import hmac
import logging
import requests
import re
import ast

from datetime import datetime

from hashlib import sha1
from urllib.parse import urlparse

from django.conf import settings
from django.contrib import messages
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.template.defaultfilters import filesizeformat

from swiftclient import client

from storage.forms import *
from storage.utils import *

from vault.jsoninfo import JsonInfo
from vault import utils
from actionlogger.actionlogger import ActionLogger


log = logging.getLogger(__name__)
actionlog = ActionLogger()


def connection(request):
    storage_url = get_storage_endpoint(request, 'adminURL')
    conn = client.http_connection(storage_url,
                                  insecure=settings.SWIFT_INSECURE,
                                  timeout=settings.SWIFT_REQUESTS_TIMEOUT)
    return storage_url, conn


@utils.project_required
@login_required
def containerview(request, project):
    """Returns a list of all containers in current account."""

    if not project:
        return redirect('change_project')

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    try:
        account_stat, containers = client.get_account(
            storage_url, auth_token, full_listing=False, http_conn=http_conn)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR,
                             _('Unable to list containers'))
        account_stat, containers = {}, []

    containers = _hide_containers_with_prefixes(containers)
    account_stat = replace_hyphens(account_stat)
    page = request.GET.get('page', 1)

    context = utils.update_default_context(request, {
        'account_stat': account_stat,
        'containers': utils.generic_pagination(containers, page),
    })

    return render(request, 'containerview.html', context)


def _hide_containers_with_prefixes(containers):
    """Hide containers that starts with prefixes listed in SWIFT_HIDE_PREFIXES."""

    hide_prefixes = settings.SWIFT_HIDE_PREFIXES
    if hide_prefixes:
        for prefix in hide_prefixes:
            containers = [
                obj for obj in containers if not obj['name'].startswith(prefix)]

    return containers


@utils.project_required
@login_required
def create_container(request, project):
    """Creates a container (empty object of type application/directory)."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    form = CreateContainerForm(request.POST or None)
    if form.is_valid():
        container = form.cleaned_data['containername']
        project_name = request.session.get('project_name')

        try:
            client.put_container(
                storage_url, auth_token, container, http_conn=http_conn)
            messages.add_message(request, messages.SUCCESS,
                                 _("Container created"))

            actionlog.log(request.user.username, "create", container)
        except client.ClientException as err:
            log.exception('Exception: {0}'.format(err))
            messages.add_message(request, messages.ERROR, _('Access denied'))

        return redirect(containerview, project=project_name)

    context = utils.update_default_context(request, {
        'form': form,
    })

    return render(request, 'create_container.html', context)


def delete_container(request, container, force=True):
    """Deletes a container. If force is True, it will deletes all objects first."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    try:
        if force:
            head = client.head_container(storage_url, auth_token, container, http_conn=http_conn)
            container_objects = []
            objects_count = int(head.get('x-container-object-count'))

            while len(container_objects) < objects_count:
                meta, objects = client.get_container(
                    storage_url, auth_token, container, marker=container_objects[-1].get("name") if
                    len(container_objects) > 0 else None, http_conn=http_conn)
                container_objects.extend(objects)

            info = get_info(storage_url)

            if 'bulk_delete' in info:
                max_deletes_per_request = info.get('bulk_delete').get('max_deletes_per_request')
                for i in range(math.ceil(len(container_objects)/max_deletes_per_request)):

                    names = b""
                    start_index = i*max_deletes_per_request
                    end_index = (i+1)*max_deletes_per_request if len(container_objects) > (i+1)*max_deletes_per_request\
                        else len(container_objects)
                    for obj in container_objects[start_index:end_index]:
                        names += prepare_data_name(container, obj['name']) + b"\n"

                    names = names[:-1]
                    headers = {"X-Auth-Token": auth_token}
                    r = requests.post(storage_url+"?bulk-delete=true",
                                      headers=headers, data=names)
            else:
                for obj in container_objects:
                    delete_object(request=request,
                                  container=container,
                                  objectname=obj['name'])

        client.delete_container(storage_url, auth_token,
                                container, http_conn=http_conn)
        actionlog.log(request.user.username, "delete", container)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        return False

    return True


def get_info(storage_url):
    info_url = storage_url.split("v1")[0] + "info"
    return json.loads(requests.get(info_url).text)


def prepare_data_name(container, obj_name):
    return container.encode() + b"/" + obj_name.encode()


@login_required
def delete_container_view(request, project, container):
    """Deletes a container."""

    if request.method != 'DELETE':
        return HttpResponse(json.dumps({'message': _('Bad parameters')}),
                            content_type='application/json',
                            status=400)

    status, content = 200, {'message': str(_('Container deleted'))}

    deleted = delete_container(request, container)
    if not deleted:
        status = 500
        msg = str(_('Container delete error'))
        content['message'] = msg
        log.error('{}. Container: {}'.format(msg, container))

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)


@utils.project_required
@login_required
def objectview(request, project, container, prefix=None):
    """Returns list of all objects in current container."""

    for hide_prefix in settings.SWIFT_HIDE_PREFIXES:
        if container.startswith(hide_prefix):
            raise Http404("Container does not exist")

    public_url = get_storage_endpoint(request, 'publicURL') + '/' + container
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    limit = settings.PAGINATION_SIZE
    marker = request.GET.get('marker')

    try:
        meta, objects = client.get_container(storage_url, auth_token,
            container, delimiter='/', prefix=prefix, full_listing=False,
            http_conn=http_conn, limit=limit, marker=marker)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        project_name = request.session.get('project_name')
        return redirect(containerview, project=project_name)

    prefixes = prefix_list(prefix)
    object_list = pseudofolder_object_list(objects, prefix, public_url)

    context = {
        'container_meta': meta,
        'container': container,
        'objects': object_list,
        'prefix': prefix,
        'prefixes': prefixes,
        'marker': None
    }

    if len(object_list) == limit:
        if prefix:
            context['marker'] = f'{objects[-1].get("name") or objects[-1].get("subdir")}'
        else:
            context['marker'] = f'{container}/{objects[-1].get("name") or objects[-1].get("subdir")}'

    return render(request, "objectview.html", context)


@utils.project_required
@login_required
def object_item(request, project, container, objectname):
    """Returns selected object in current container."""

    storage_url = get_storage_endpoint(request, 'adminURL')
    headers = {'X-Storage-Token': get_token_id(request)}
    custom_headers = {}
    system_headers = {}

    url = '{0}/{1}'.format(storage_url, container)
    if objectname:
        url = '{0}/{1}'.format(url, str(objectname))

    response = requests.head(url, headers=headers,
                             verify=not settings.SWIFT_INSECURE)

    metadata = dict(response.headers)

    if 'Cache-Control' not in metadata.keys():
        metadata["Cache-Control"] = ''

    public_url = '{}/{}/{}'.format(get_storage_endpoint(request,
                                   'publicURL'), container, objectname)
    prefixes = prefix_list(objectname)

    for item in metadata:
        if 'x-object-meta-' in item.lower():
            custom_headers[re.sub('x-object-meta-', '',
                                  item.lower())] = metadata[item]
        else:
            system_headers[item] = metadata[item]

    context = utils.update_default_context(request, {
        'project': project,
        'container': container,
        'objectname': objectname,
        'public_url': public_url,
        'custom_headers': custom_headers,
        'system_headers': system_headers,
        'prefix': objectname,
        'prefixes': prefixes,
    })

    return render(request, "object.html", context)


@login_required
def upload(request, project, container, prefix=None):
    """Display upload form using swift formpost."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    project_name = request.session.get('project_name')
    redirect_url = 'http://{}'.format(request.get_host())
    redirect_url += reverse('objectview',
        kwargs={'container': container, 'project': project_name})

    swift_url = storage_url + '/' + container + '/'
    if prefix:
        swift_url += prefix
        redirect_url += prefix

    url_parts = urlparse(swift_url)
    path = url_parts.path

    max_file_size = 5 * 1024 * 1024 * 1024
    max_file_count = settings.MAX_FILES_UPLOAD
    expires = int(time.time() + 15 * 60)
    key = get_temp_key(storage_url, auth_token, http_conn)

    if not key:
        messages.add_message(request, messages.ERROR, _('Access denied'))
        if prefix:
            return redirect(objectview, container=container, prefix=prefix, project=project_name)
        else:
            return redirect(objectview, container=container, project=project_name)

    hmac_body = '{}\n{}\n{}\n{}\n{}'.format(
        path, redirect_url, max_file_size, max_file_count, expires)
    signature = hmac.new(
        bytes(key, 'utf-8'), bytes(hmac_body, 'utf-8'), sha1).hexdigest()

    prefixes = prefix_list(prefix)

    context = utils.update_default_context(request, {
        'swift_url': swift_url,
        'redirect_url': redirect_url,
        'max_file_size': max_file_size,
        'max_file_count': max_file_count,
        'expires': expires,
        'signature': signature,
        'container': container,
        'prefix': prefix if prefix is not None else '',
        'prefixes': prefixes,
    })

    return render(request, 'upload_form.html', context)


@login_required
def create_object(request, project, container, prefix=None):
    """Create object on Swift."""

    obj = request.FILES.get('file1')
    project_name = request.session.get('project_name')

    if obj:
        content = obj.read()
        content_type = obj.content_type

        storage_url = get_storage_endpoint(request, 'adminURL')
        auth_token = get_token_id(request)

        storage_url += '/' + container + '/'

        if prefix:
            storage_url += prefix

        storage_url += obj.name

        req = requests.put(storage_url, data=content,
            headers={'X-Storage-Token': auth_token, 'content-type': content_type},
            verify=not settings.SWIFT_INSECURE)

        if req.status_code == 201:
            messages.add_message(
                request, messages.SUCCESS, _('Object created'))
            actionlog.log(request.user.username, "create", obj)
        elif req.status_code == 401 or req.status_code == 403:
            messages.add_message(request, messages.ERROR, _('Access denied'))
        else:
            msg = 'Fail to create object ({0}).'.format(req.status_code)
            log.error(msg)
            messages.add_message(request, messages.ERROR, msg)

    if prefix:
        return redirect(objectview, container=container, prefix=prefix, project=project_name)
    else:
        return redirect(objectview, container=container, project=project_name)


def download(request, project, container, objectname):
    """Download an object from Swift."""

    storage_url = get_storage_endpoint(request, 'adminURL')
    auth_token = get_token_id(request)

    headers = {'X-Storage-Token': auth_token}

    url = '{0}/{1}/{2}'.format(storage_url, container, str(objectname))

    res = requests.get(
        url, headers=headers, verify=not settings.SWIFT_INSECURE)

    actionlog.log(request.user.username, "download", str(objectname))

    return HttpResponse(res.content, content_type=res.headers['Content-Type'])


@login_required
def delete_object_view(request, project, container, objectname):
    """Deletes an object."""

    response = delete_object(request, container, objectname)

    if response:
        messages.add_message(request, messages.SUCCESS, _('Object deleted'))
    else:
        messages.add_message(request, messages.ERROR, _('Access denied'))

    prefix = '/'.join(objectname.split('/')[:-1])
    project_name = request.session.get('project_name')

    if prefix:
        prefix += '/'
        return redirect(objectview, container=container, prefix=prefix, project=project_name)
    else:
        return redirect(objectview, container=container, project=project_name)


def delete_object(request, container, objectname):
    """Deletes an object from swift."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    try:
        client.delete_object(storage_url, token=auth_token,
            container=container, name=objectname, http_conn=http_conn)
        actionlog.log(request.user.username, "delete", objectname)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        return False

    return True


@login_required
def delete_pseudofolder(request, project, container, pseudofolder):
    """Deletes an empty object, used as a pseudofolder."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    objects = client.get_container(storage_url, auth_token,
        container, prefix=pseudofolder, http_conn=http_conn)[1]

    count_deletes = 0

    for obj in objects:
        try:
            client.delete_object(storage_url, token=auth_token,
                container=container, name=obj['name'], http_conn=http_conn)
            count_deletes += 1
        except client.ClientException as err:
            log.exception('Exception: {0}'.format(err))

    # Empty pseudofolder
    if count_deletes == 1 and count_deletes == len(objects):
        messages.add_message(request, messages.SUCCESS, _('Pseudofolder deleted'))

    # Non empty pseudofolder
    elif count_deletes > 1 and count_deletes == len(objects):
        messages.add_message(request, messages.SUCCESS,
                             'Pseudofolder and {0} objects deleted.'.format(count_deletes - 1))
    elif count_deletes > 0 and count_deletes < len(objects):
        messages.add_message(request, messages.SUCCESS,
                             _('Could not delete all objects'))
    else:
        messages.add_message(request, messages.ERROR,
                             _('Fail to delete pseudofolder'))

    if pseudofolder[-1] == '/':  # deleting a pseudofolder, move one level up
        pseudofolder = pseudofolder[:-1]

    prefix = '/'.join(pseudofolder.split('/')[:-1])

    if prefix:
        prefix += '/'

    project_name = request.session.get('project_name')
    actionlog.log(request.user.username, "delete", pseudofolder)

    if prefix:
        return redirect(objectview, container=container, prefix=prefix, project=project_name)
    else:
        return redirect(objectview, container=container, project=project_name)


@login_required
def create_pseudofolder(request, project, container, prefix=None):
    """Creates a pseudofolder (empty object of type application/directory)."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    form = PseudoFolderForm(request.POST or None)

    if form.is_valid():
        foldername = request.POST.get('foldername', None)
        if prefix:
            foldername = prefix + '/' + foldername
        foldername = os.path.normpath(foldername)
        foldername = foldername.strip('/')
        foldername += '/'

        content_type = 'application/directory'
        obj = None
        project_name = request.session.get('project_name')

        try:
            client.put_object(storage_url, auth_token, container,
                foldername, obj, content_type=content_type, http_conn=http_conn)
            messages.add_message(request, messages.SUCCESS,
                                 _('Pseudofolder created'))
        except client.ClientException as err:
            log.exception('Exception: {0}'.format(err))
            messages.add_message(request, messages.ERROR, _('Access denied'))

        if prefix:
            actionlog.log(request.user.username, "create", foldername)
            return redirect(objectview, container=container, prefix=prefix, project=project_name)

        actionlog.log(request.user.username, "create", foldername)
        return redirect(objectview, container=container, project=project_name)

    prefixes = prefix_list(prefix)

    context = utils.update_default_context(request, {
        'container': container,
        'prefix': prefix,
        'prefixes': prefixes,
        'form': form,
    })

    return render(request, 'create_pseudofolder.html', context)


@login_required
def edit_acl(request, project, container):
    """Edit ACLs on given container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    if request.method == 'POST':
        form = AddACLForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']

            readers, writers = get_acls(
                storage_url, auth_token, container, http_conn)

            readers = remove_duplicates_from_acl(readers)
            writers = remove_duplicates_from_acl(writers)

            if form.cleaned_data['read']:
                readers += ",{}".format(username)

            if form.cleaned_data['write']:
                writers += ",{}".format(username)

            headers = {'X-Container-Read': readers,
                       'X-Container-Write': writers}
            try:
                client.post_container(storage_url,
                    auth_token, container, headers=headers, http_conn=http_conn)

                messages.add_message(request, messages.SUCCESS, _('ACLs updated'))

                actionlog.log(request.user.username, "update",
                              'headers: {}, container: {}'.format(headers, container))

            except client.ClientException as err:
                log.exception('Exception: {0}'.format(err))
                messages.add_message(request, messages.ERROR,
                                     _('ACL update failed'))

    if request.method == 'GET':
        delete = request.GET.get('delete', None)
        if delete:
            users = delete.split(',')

            (readers, writers) = get_acls(storage_url, auth_token,
                                          container, http_conn)

            new_readers = ''
            for element in readers.split(','):
                if element not in users:
                    new_readers += element
                    new_readers += ","

            new_writers = ''
            for element in writers.split(','):
                if element not in users:
                    new_writers += element
                    new_writers += ","

            headers = {'X-Container-Read': new_readers,
                       'X-Container-Write': new_writers}
            try:
                client.post_container(storage_url, auth_token,
                                      container, headers=headers, http_conn=http_conn)

                messages.add_message(request, messages.SUCCESS,
                                     _('ACL removed'))

                actionlog.log(request.user.username, "delete",
                              'headers: {}, container: {}'.format(headers, container))

            except client.ClientException as err:
                log.exception('Exception: {0}'.format(err))
                messages.add_message(request, messages.ERROR,
                                     _('ACL update failed'))

    readers, writers = get_acls(
        storage_url, auth_token, container, http_conn)

    acls = {}

    if readers != '':
        readers = remove_duplicates_from_acl(readers)
        for entry in readers.split(','):
            acls[entry] = {}
            acls[entry]['read'] = True
            acls[entry]['write'] = False

    if writers != '':
        writers = remove_duplicates_from_acl(writers)
        for entry in writers.split(','):
            if entry not in acls:
                acls[entry] = {}
                acls[entry]['read'] = False
            acls[entry]['write'] = True

    public = False
    if acls.get('.r:*', False):
        public = True

    context = utils.update_default_context(request, {
        'container': container,
        'session': request.session,
        'acls': acls,
        'public': public
    })

    return render(request, 'edit_acl.html', context)


@login_required
def make_public(request, container):
    """Enable/Disable public access to a container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    headers = client.head_container(
        storage_url, auth_token, container, http_conn=http_conn)

    current = headers.get('X-Container-Read')
    if current is None:
        headers['X-Container-Read'] = '.r:*'
        message = _('Container is now public')
    else:
        del headers['X-Container-Read']
        message = _('Container is now private')

    try:
        client.post_container(storage_url,
            auth_token, container, headers=headers, http_conn=http_conn)

        content = {"message": str(message)}
        msg = "X-Container-Read header on container {}".format(container)
        actionlog.log(request.user.username, "update", msg)
    except client.ClientException as err:
        content, status = {"message": str(
            _("Container ACL update failed"))}, 500
        log.exception("Exception: {}".format(err))

    return HttpResponse(
        json.dumps(content), content_type='application/json', status=status)


@login_required
def metadataview(request, project, container, objectname=None):
    """Return object/container/pseudofolder metadata."""

    storage_url = get_storage_endpoint(request, 'adminURL')
    headers = {'X-Storage-Token': get_token_id(request)}

    url = '{0}/{1}'.format(storage_url, container)
    if objectname:
        url = '{0}/{1}'.format(url, str(objectname))

    response = requests.head(
        url, headers=headers, verify=not settings.SWIFT_INSECURE)

    content = json.dumps(dict(response.headers))

    status = response.status_code
    if status >= 200 and status < 300 or status == 304:
        status = 200
    else:
        status = 404

    return HttpResponse(
        content, content_type='application/json', status=status)


@login_required
def object_versioning(request, project, container, prefix=None):
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)
    public_url = get_storage_endpoint(request, 'publicURL') + '/' + container

    objects = []

    page = request.GET.get('page', 1)

    if request.method == 'GET':
        headers = client.head_container(
            storage_url, auth_token, container, http_conn=http_conn)

        version_location = headers.get('x-versions-location', None)

        if version_location:
            try:
                meta, objects = client.get_container(
                    storage_url, auth_token, version_location, prefix=prefix,
                    delimiter='/', full_listing=False, http_conn=http_conn)
            except client.ClientException:
                pass

        prefixes = prefix_list(prefix)
        object_list = pseudofolder_object_list(objects, prefix, public_url)

        context = utils.update_default_context(request, {
            'container': container,
            'objects': utils.generic_pagination(object_list, page),
            'version_location': version_location,
            'prefix': prefix,
            'prefixes': prefixes,
        })

        return render(request, 'container_versioning.html', context)

    if request.method == 'POST':

        action = request.POST.get('action', None)

        if action == 'enable':
            enable_versioning(request, container)
            actionlog.log(request.user.username, "enable",
                          'Versioning. Container: {}'.format(container))
        elif action == 'disable':
            disable_versioning(request, container)
            actionlog.log(request.user.username, "disable",
                          'Versioning. Container: {}'.format(container))
        else:
            messages.add_message(request, messages.ERROR,
                                 'Action is required.')

        return redirect(object_versioning, project=project, container=container)


def enable_versioning(request, container):
    """Enable/Disable versioning in container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    version_location = '{0}{1}'.format(
        settings.SWIFT_VERSION_PREFIX, container)

    try:
        client.put_container(
            storage_url, auth_token, version_location, http_conn=http_conn)
        actionlog.log(request.user.username, "create", version_location)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        return False

    try:
        header = {'x-versions-location': version_location}
        client.post_container(storage_url,
            auth_token, container, headers=header, http_conn=http_conn)
        actionlog.log(request.user.username, "update", version_location)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        return False

    messages.add_message(request, messages.SUCCESS, _('Versioning enabled'))

    return True


def disable_versioning(request, container):
    """Enable/Disable versioning in container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    try:
        headers = client.head_container(
            storage_url, auth_token, container, http_conn=http_conn)
    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        return False

    version_location = headers.get('x-versions-location', None)

    if version_location:
        try:
            client.post_container(storage_url, auth_token, container,
                headers={'x-versions-location': ''}, http_conn=http_conn)
            actionlog.log(request.user.username, "update", container)
        except client.ClientException as err:
            log.exception('Exception: {0}'.format(err))
            messages.add_message(request, messages.ERROR, _('Access denied'))
            return False

        deleted = delete_container(request=request, container=version_location)
        if not deleted:
            return False

    messages.add_message(request, messages.SUCCESS, _('Versioning disabled'))

    return True


@login_required
def edit_cors(request, project, container):
    """Edit CORS on given container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    if request.method == 'POST':
        form = AddCORSForm(request.POST)
        if form.is_valid():
            cors = get_cors(
                storage_url, auth_token, container, http_conn)

            cors = remove_duplicates_from_cors(cors)

            host = form.cleaned_data['host']
            if host:
                cors += " {}".format(host)

            headers = {
                'x-container-meta-access-control-allow-origin': cors.strip()
            }

            try:
                client.post_container(storage_url,
                    auth_token, container, headers=headers, http_conn=http_conn)

                messages.add_message(request, messages.SUCCESS, _('CORS updated'))

                actionlog.log(request.user.username, "update",
                              'headers: {}, container: {}'.format(headers, container))

            except client.ClientException as err:
                log.exception('Exception: {0}'.format(err))
                messages.add_message(request, messages.ERROR, _('CORS update failed'))

    if request.method == 'GET':
        delete = request.GET.get('delete', None)
        if delete:
            host = delete.split(' ')
            cors = get_cors(storage_url, auth_token, container, http_conn)

            new_cors = ''
            for element in cors.split(' '):
                if element not in host:
                    new_cors += element
                    new_cors += ' '

            headers = {
                'x-container-meta-access-control-allow-origin': new_cors.strip()
            }

            try:
                client.post_container(storage_url,
                    auth_token, container, headers=headers, http_conn=http_conn)

                messages.add_message(request, messages.SUCCESS, _('CORS removed'))

                actionlog.log(request.user.username, "delete",
                              'headers: {}, container: {}'.format(headers, container))

            except client.ClientException as err:
                log.exception('Exception: {0}'.format(err))
                messages.add_message(request, messages.ERROR, _('CORS update failed'))

    cors = get_cors(storage_url, auth_token, container, http_conn)

    context = utils.update_default_context(request, {
        'container': container,
        'session': request.session,
        'cors': [],
    })

    if cors != '':
        cors = remove_duplicates_from_cors(cors)
        for entry in cors.split(' '):
            context['cors'].append(entry)

    return render(request, 'edit_cors.html', context)


@utils.project_required
@login_required
def edit_custom_metadata(request, project, container, objectname):
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    content, status = {}, 200
    custom_headers = request.POST.dict()
    system_headers = {}

    headers = client.head_object(storage_url,
        auth_token, container, objectname, http_conn=http_conn)

    for item in headers:
        if 'x-object-meta-' not in item.lower():
            system_headers[item] = headers[item]

    system_headers.update(custom_headers)

    try:
        client.post_object(storage_url, auth_token, container,
            objectname, headers=system_headers, http_conn=http_conn)

        content = {"message": _("Custom Metadata updated")}
        msg = "Custom Metadata header on object {}/{}".format(container, objectname)
        actionlog.log(request.user.username, "update", msg)
    except client.ClientException as err:
        content, status = {"message": str(
            _("Custom Metadata update failed"))}, 500
        log.exception("Exception: {}".format(err))

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@login_required
def cache_control(request, project, container, objectname):
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    content, status = {}, 200
    unit = request.POST.get("unit", "minutes")
    maxage = int(request.POST.get("maxage", 3)) * 60

    if maxage <= 0:
        content, status = {"message": _("Days must be greater than 0")}, 400
        return HttpResponse(
            json.dumps(content), content_type='application/json', status=status)

    if unit == "hours":
        maxage = maxage * 3600
    elif unit == "days":
        maxage = maxage * 86400

    headers = client.head_object(storage_url,
        auth_token, container, objectname, http_conn=http_conn)
    headers["cache-control"] = "public, max-age={}".format(maxage)

    try:
        client.post_object(storage_url, auth_token,
            container, objectname, headers=headers, http_conn=http_conn)
        content = {"message": str(
            _("Cache-Control updated")), "cache_control": headers["cache-control"]}
        msg = "Cache-Control header on object {}/{}".format(container,
                                                            objectname)
        actionlog.log(request.user.username, "update", msg)
    except client.ClientException as err:
        content, status = {"message": str(
            _("Cache-Control update failed"))}, 500
        log.exception("Exception: {}".format(err))

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@login_required
def optional_headers(request, project, container, objectname):
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    content, status = {}, 200

    headers = client.head_object(storage_url,
        auth_token, container, objectname, http_conn=http_conn)

    body_dict = ast.literal_eval(request.body.decode())
    headers['x-delete-at'] = body_dict.get('x-delete-at')
    log.error(headers['x-delete-at'])
    if (headers['x-delete-at'] == ''):
        headers.pop('x-delete-at', None)

    content = {"message": str(_("Optional headers updated")),
               "x-delete-at": headers.get("x-delete-at")}
    try:
        client.post_object(storage_url, auth_token, container, objectname,
                           headers=headers, http_conn=http_conn)
    except Exception as e:
        log.error(e)
        content, status = {"message": str(
            _("Optional headers update failed: {}".format(e)))}, 500

    return HttpResponse(json.dumps(content),
                        content_type='application/json',
                        status=status)


@utils.project_required
@login_required
def get_deleted_objects(request, project, container, prefix=None):
    """Return all deleted objects from a given container."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)
    public_url = get_storage_endpoint(request, 'publicURL')
    trash_container = "{}-{}".format(settings.SWIFT_TRASH_PREFIX, container)
    objects = None

    try:
        meta, objects = client.get_container(storage_url,
            auth_token, trash_container, delimiter='', http_conn=http_conn)
    except client.ClientException as err:
        if err.http_status == 404:
            log.info("Not found: {}".format(trash_container))
            objects = []
        else:
            log.exception("Exception: {0}".format(err.msg))
            status = err.http_status
            content = {"error": err.msg}

    if objects is not None:
        object_list = pseudofolder_object_list(objects, prefix, public_url)

        deleted = []
        for item in object_list:
            if 'prefix' not in item:
                name = item.get('name')
                deleted.append({
                    "name": name,
                    "size": item.get('bytes')
                })

        status = 200
        content = {
            "deleted_objects": deleted,
            "prefix": prefix,
            "storage_url": storage_url,
            "original_container": container,
            "trash_container": trash_container
        }

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@utils.project_required
@login_required
def restore_object(request, project):
    """Restore a object from trash."""

    if request.method != 'POST':
        return HttpResponse(status=405)

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    container = request.POST.get("container")
    trash_container = request.POST.get("trash_container")
    object_name = request.POST.get("object_name")
    object_new_name = request.POST.get("object_new_name")

    content = {}
    status = 200

    # valida se objeto a ser restaurado da lixeira já existe no container.
    try:
        if object_new_name != '':
            client.head_object(storage_url,
                auth_token, container, object_new_name, http_conn=http_conn)
        else:
            client.head_object(storage_url,
                auth_token, container, object_name, http_conn=http_conn)

        status = 409

        # Novo nome será o nome original + timestamp + extensão
        new_object_name = object_name.rsplit('.', 1)
        orignal_name = new_object_name[0] + '_'
        time_stamp = datetime.isoformat(datetime.now())
        extension_object = '.' + new_object_name[1]

        final_new_object_name = orignal_name + time_stamp + extension_object
        content = {
            'original_object_name': object_name,
            'new_object_name': final_new_object_name
        }

    except client.ClientException as err:
        # Segue o fluxo normal se objeto a ser restaurado não foi localizado no container
        obj_headers = client.head_object(storage_url,
            auth_token, trash_container, object_name, http_conn=http_conn)
        custom_headers = {
            "X-Fresh-Metadata": "True",
            "X-Copy-From": "/{}/{}".format(trash_container, object_name)
        }
        for key, value in obj_headers.items():
            if "x-object-meta" in key:
                custom_headers[key] = value

        # Upload do objeto para o container de origem
        try:
            if object_new_name != '':
                client.put_object(storage_url, auth_token, container, object_new_name,
                    content_length=0, headers=custom_headers, http_conn=http_conn)
            else:
                client.put_object(storage_url, auth_token, container, object_name,
                    content_length=0, headers=custom_headers, http_conn=http_conn)
            messages.add_message(request, messages.SUCCESS,
                                 _('Object restored'))
            actionlog.log(request.user.username, "restore", object_name)
        except client.ClientException as err:
            log.exception('Exception: {0}'.format(err.msg))
            status = err.http_status
            content = {"error": err.msg}

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@utils.project_required
@login_required
def remove_from_trash(request, project):

    if request.method != 'POST':
        return HttpResponse(status=405)

    trash_container = request.POST.get("trash_container")
    object_name = request.POST.get("object_name")

    result = delete_object(request, trash_container, object_name)

    status = 202
    content = {}

    if not result:
        status = 500
        content = {
            "error": "Object '{}' was not removed from trash".format(object_name)
        }

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@utils.project_required
@login_required
def config_trash_container(request, project, container):
    """Enable or disable container trash with the metadata X-Undelete-Enabled configured to True or False."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    status, content = 200, {'message': ''}
    enable = False
    content['message'] = (
        '{} "{}"'.format(_('Trash disabled for container'), container)
    )

    try:
        if request.GET.get('status').lower() == 'enabled':
            enable = True
            content['message'] = (
                '{} "{}"'.format(_('Trash enabled for container'), container)
            )

        client.put_container(storage_url, auth_token, container,
            headers={'X-Storage-Token': auth_token, 'X-Undelete-Enabled': str(enable)},
            http_conn=http_conn)

        # messages.add_message(request, messages.SUCCESS, msg)
        actionlog.log(request.user.username, 'update_trash', container)
    except Exception as err:
        log.exception('Exception: {0}'.format(err))
        # messages.add_message(request, messages.ERROR, _('Update container trash error'))
        status, content = 500, {'message': 'Update container trash error'}

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@utils.project_required
@login_required
def container_trash_status(request, project, container):
    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    status, content = 200, {'status': 'enabled'}
    try:
        headers = client.head_container(storage_url,
            auth_token, container, http_conn=http_conn)

        undelete_enabled = headers.get('x-undelete-enabled')
        if undelete_enabled is None or undelete_enabled == 'False':
            content['status'] = 'disabled'

    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        status, content = 500, {'message': 'Access denied'}

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@login_required
def container_acl_update(request, project, container):
    """Sets a container to public or private."""

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    headers = client.head_container(storage_url,
        auth_token, container, http_conn=http_conn)

    msg = ''
    public = False
    if request.GET.get('status').lower() == 'enabled':
        headers['x-container-read'] = '.r:*'
        msg = _('Container is now public')
        public = True
    else:
        headers['x-container-read'] = ''
        msg = str(_('Container is now private'))

    status, content = 200, {'message': str(msg)}

    try:
        client.post_container(storage_url,
            auth_token, container, headers=headers, http_conn=http_conn)
        if public:
            actionlog.log(request.user.username, "set_public", container)
        else:
            actionlog.log(request.user.username, "set_private", container)
    except Exception as err:
        log.exception('Exception: {0}'.format(err))
        status, content = 500, {'message': 'Failed on update container ACL'}

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


@utils.project_required
@login_required
def container_acl_status(request, project, container):
    """
    Makes a head in a container and returns to the context
    status = enabled if it's public, status = disabled for private
    """

    auth_token = get_token_id(request)
    storage_url, http_conn = connection(request)

    status, content = 200, {'status': 'disabled'}
    try:
        headers = client.head_container(storage_url,
            auth_token, container, http_conn=http_conn)

        acl_header = headers.get('x-container-read', '')
        if '.r:*' in acl_header:
            content['status'] = 'enabled'

    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        messages.add_message(request, messages.ERROR, _('Access denied'))
        status, content = 500, {'message': 'Access denied'}

    return HttpResponse(json.dumps(content),
        content_type='application/json', status=status)


class SwiftJsonInfo(JsonInfo):

    def generate_menu_info(self):
        project_name = self.request.session.get('project_name')
        menu = {
            "name": str(_("Object Storage")),
            "icon": "fas fa-cube",
            "url": reverse("containerview", kwargs={'project': project_name}),
            "subitems": [
                # {
                #     "name": str(_("Account")),
                #     "icon": "",
                #     "url": reverse("accountview", kwargs={'project': project_name})
                # },
                {
                    "name": str(_("Containers")),
                    "icon": "",
                    "url": reverse("containerview", kwargs={'project': project_name})
                }
            ]
        }

        if settings.CACHE_APIS_ENABLED:
            menu["subitems"].append({
                "name": str(_("Cache")),
                "icon": "",
                "url": reverse("storage_cache", kwargs={'project': project_name})
            })

        return menu

    def _get_cloud_status(self):
        if not settings.SWIFT_CLOUD_TOOLS_URL:
            return ""

        project_id = self.request.session.get('project_id')
        status = ""
        try:
            url = f"{settings.SWIFT_CLOUD_TOOLS_URL}/transfer/status/{project_id}"
            headers = {"X-Auth-Token": settings.SWIFT_CLOUD_TOOLS_API_KEY}
            res = requests.get(url, headers=headers)
            if res.status_code == 200:
                data = res.json()
                status = data.get('status')
                if data.get('progress'):
                    status = f"{status} {data.get('progress')}%"
        except Exception as err:
            log.exception(err)

        return status

    def generate_widget_info(self):
        storage_url, http_conn = connection(self.request)
        auth_token = self.request.session.get('auth_token')
        project_name = self.request.session.get('project_name')

        if storage_url is None:
            return {"error": "Storage URL not found."}

        try:
            head_acc = client.head_account(
                storage_url, auth_token, http_conn=http_conn)
        except Exception as err:
            log.exception('Exception: {0}'.format(err))
            return {"error": "Unable to show Swift info."}

        widget_info = [{
            "type": "default",
            "name": "storage",
            "title": str(_("Swift")),
            "subtitle": str(_("Object Storage")),
            "color": "red",
            "icon": "fas fa-cube",
            "url": reverse("containerview", kwargs={'project': project_name}),
            "properties": [
                {
                    "name": str(_("containers")),
                    "description": "",
                    "value": head_acc.get('x-account-container-count')
                },
                {
                    "name": str(_("objects")),
                    "description": "",
                    "value": head_acc.get('x-account-object-count')
                },
                {
                    "name": str(_("used space")),
                    "description": "",
                    "value": filesizeformat(head_acc.get('x-account-bytes-used'))
                }
            ],
            "buttons": [
                {
                    "name": str(_("Containers")),
                    "url": reverse("containerview", kwargs={'project': project_name})
                }
            ]
        }]

        if settings.CACHE_APIS_ENABLED:
            widget_info[0]['buttons'].append({
                "name": "Cache",
                "url": reverse("storage_cache", kwargs={'project': project_name})
            })

        if 'x-account-meta-cloud' in head_acc:
            widget_info[0]['extra'] = {
                "icon": "fas fa-cloud",
                "title": str(_("This project saves its objects in a public cloud")),
                "status": self._get_cloud_status()
            }

        return widget_info


@utils.project_required
@login_required
def info_json(request, project=None):
    sjinfo = SwiftJsonInfo(request=request)
    return sjinfo.render(request)
